using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public abstract class CmsControllerBase
    {

        public CmsControllerBase(IObjectFactory factory, ICmsFolderService folderService)
        {

            if (folderService == null)
                throw new ArgumentNullException("folderService", "folderService is null.");

            m_folderServiceInstance = folderService;

            if (factory == null)
            {
                throw new ArgumentNullException("factory", "Argument cannot be null.");
            }

            m_objectFactoryInstance = factory;
        }
        private IObjectFactory m_objectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get
            {
                return m_objectFactoryInstance;
            }
        }
        private ICmsFolderService m_folderServiceInstance;
        protected ICmsFolderService FolderServiceInstance
        {
            get
            {
                return m_folderServiceInstance;
            }
        }
        private void PopulateSubfoldersSortedAndGrouped(IList<IFolderInfo> allFolders, IFolderInfo populateThis)
        {
            if (allFolders == null)
                throw new ArgumentNullException("allFolders", "allFolders is null.");
            if (populateThis == null)
                throw new ArgumentNullException("populateThis", "populateThis is null.");
            IList<IFolderInfo> grouped = new List<IFolderInfo>();
            var subfolders = from temp in allFolders 
                             where temp.ParentFolderId == populateThis.Id 
                             orderby temp.DisplayOrder 
                             select temp;

            foreach (var subfolder in subfolders)
            {
                PopulateSubfoldersSortedAndGrouped(allFolders, subfolder);
                grouped.Add(subfolder);
            }
            populateThis.Folders = grouped;
        }

        protected IList<IFolderInfo> SortAndGroupFolders(IList<IFolderInfo> allFolders)
        {
            IList<IFolderInfo> grouped = new List<IFolderInfo>();
            var rootFolders = from temp in allFolders
                              where BusinessUtility.IsItemIdSaved(temp.ParentFolderId) == false
                              orderby temp.DisplayOrder
                              select temp;

            foreach (var rootFolder in rootFolders)
            {
                PopulateSubfoldersSortedAndGrouped(allFolders, rootFolder);
                grouped.Add(rootFolder);
            }
            return grouped;
        }        
    }
}
