using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;
using System.Configuration;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class CmsNavigationItemAdapter
    {
        public CmsNavigationItemAdapter()
        {

        }

        private static ICmsNavigationItem Adapt(ICmsFolder folder)
        {
            CmsNavigationItem temp = new CmsNavigationItem();

            temp.Id = folder.Id;
            temp.Name = folder.Name;
            temp.Key = folder.Key;

            var defaultItem = folder.FindDefaultItem();

            if (defaultItem != null)
            {
                temp.TargetKey = defaultItem.Key;
                PopulateUrlForNamedItem(defaultItem, temp);
            }
            else
            {
                temp.TargetKey = folder.Key;
                temp.Url = String.Empty;
            }            

            return temp;
        }

        private static string m_linkToPageTemplate;
        private static string LinkToPageTemplate
        {
            get
            {
                if (m_linkToPageTemplate == null)
                {
                    m_linkToPageTemplate = ConfigurationManager.AppSettings["WEB_SITE.MENU.LINK_TO_ITEM_TEMPLATE"];
                }

                return m_linkToPageTemplate;
            }
        }

        private static void PopulateUrlForNamedItem(ICmsNamedItem item, CmsNavigationItem temp)
        {
            if (item is ICmsLink)
            {
                temp.Url = (item as ICmsLink).Url;
            }
            else if (item is ICmsWebPage)
            {
                temp.Url = String.Format(LinkToPageTemplate, item.Id);
            }
        }

        private static ICmsNavigationItem Adapt(ICmsNamedItem item)
        {
            CmsNavigationItem temp = new CmsNavigationItem();

            temp.Id = item.Id;
            temp.Name = item.Name;
            temp.Key = item.Key;
            temp.TargetKey = temp.Key;

            PopulateUrlForNamedItem(item, temp);

            return temp;
        }
        public static IList<ICmsNavigationItem> Adapt(IList<ICmsNamedItem> namedItems)
        {
            List<ICmsNavigationItem> list = new List<ICmsNavigationItem>();

            foreach (var item in namedItems)
            {
                list.Add(Adapt(item));
            }

            return list;
        }

        public static IList<ICmsNavigationItem> Adapt(IList<ICmsFolder> folders)
        {
            List<ICmsNavigationItem> list = new List<ICmsNavigationItem>();

            foreach (var item in folders)
            {
                list.Add(Adapt(item));
            }

            return list;
        }
    }
}
