using System;
using System.Collections.Generic;
using System.Security.Principal;
using Benday.Cms.Business.Interfaces;
using System.Configuration;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public abstract class DomainObjectService<T> : IDomainObjectService<T>
                where T : IBusinessBase
    {
        private IRepository<T> m_repositoryInstance;
        protected IRepository<T> RepositoryInstance
        {
            get { return m_repositoryInstance; }
        }

        public DomainObjectService(IRepository<T> repository)
        {
            if (repository == null)
            {
                throw new ArgumentNullException("repository", "Argument cannot be null.");
            }

            m_repositoryInstance = repository;
        }

        public virtual IList<T> GetList()
        {
            return RepositoryInstance.GetAll();
        }

        public virtual T GetById(int id)
        {
            return RepositoryInstance.GetById(id);
        }

        private void UpdateCreatedInfo(T saveThis)
        {
            saveThis.CreateDate = DateTime.Now;
            saveThis.CreatedBy = WindowsIdentity.GetCurrent().Name;
        }

        private void UpdateModifiedInfo(T saveThis)
        {
            if (ConfigurationManager.AppSettings["DomainObjectService.SkipUpdateLastModified"] != "true")
            {
                saveThis.LastModified = DateTime.Now;    
            }
            
            saveThis.LastModifiedBy = WindowsIdentity.GetCurrent().Name;
        }
        
        protected virtual void BeforeSave(T saveThis)
        {
            if (saveThis == null)
            {
                return;
            }
            else
            {
                if (BusinessUtility.IsItemIdSaved(saveThis.Id) == false)
                {
                    UpdateCreatedInfo(saveThis);
                }

                UpdateModifiedInfo(saveThis);
            }            
        }

        protected virtual void AfterSave(T saveThis)
        {
            
        }

        public virtual void Save(T saveThis)
        {
            if (saveThis == null)
                throw new ArgumentNullException("saveThis", "saveThis is null.");
         
            BeforeSave(saveThis);
            RepositoryInstance.Save(saveThis);
            AfterSave(saveThis);
        }

        protected virtual void BeforeDelete(T saveThis)
        {
        }

        protected virtual void AfterDelete(T saveThis)
        {            
        }

        public virtual void Delete(T deleteThis)
        {
            BeforeDelete(deleteThis);
            RepositoryInstance.Delete(deleteThis);
            AfterDelete(deleteThis);
        }
    }
}
