using System;
using Microsoft.Practices.Unity;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class FileController : FolderItemController, IFileController
    {
        [InjectionConstructor]
        public FileController(ICmsFolderService folderService, ICmsFileService fileService, IObjectFactory factory)
            : base(factory, folderService)
        {
            if (fileService == null)
            {
                throw new ArgumentNullException("fileService", "Argument cannot be null.");
            }

            m_fileServiceInstance = fileService;
        }

        private ICmsFileService m_fileServiceInstance;
        protected ICmsFileService FileServiceInstance
        {
            get
            {
                return m_fileServiceInstance;
            }
        }

        public void InitializeBlank(IFileDetailView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            view.Filename = String.Empty;
            view.IsImage = true;
            view.FileData = new byte[] { };

            InitializeBlankFolderItemView(view);
        }

        public void Save(IFileDetailView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsFile file = null;

            if (BusinessUtility.IsItemIdSaved(view.ItemId) == true)
            {
                file = FileServiceInstance.GetById(view.ItemId);
                ControllerUtility.AssertItemWasLoaded(file, "link", view.ItemId);
            }
            else
            {
                file = ObjectFactoryInstance.Create<ICmsFile>();

                file.Status = "ACTIVE";
            }

            if (BusinessUtility.IsItemIdSaved(view.ParentFolderId) == false)
            {
                throw new ValidationException("ParentFolderId", "You must specify a valid parent folder id.");
            }
            else
            {
                var parentFolder = FolderServiceInstance.GetById(view.ParentFolderId);

                if (parentFolder == null)
                {
                    throw new UnknownIdException("folder", view.ParentFolderId);
                }
                else
                {
                    file.Parent = parentFolder;
                }
            }

            file.Name = view.Name;
            file.DisplayOrder = view.DisplayOrder;
            file.IsVisible = view.IsVisible;
            file.Filename = view.Filename;
            file.IsImage = view.IsImage;
            file.Filedata.Data = view.FileData;

            FileServiceInstance.Save(file);

            view.ItemId = file.Id;

            if (view.ModifyFile == true)
            {
                view.Filename = view.Filename;
                view.ModifyFile = false;
            }
        }

        public void DeleteById(int linkId)
        {
            var link = FileServiceInstance.GetById(linkId);

            ControllerUtility.AssertItemWasLoaded(link, "link", linkId);

            FileServiceInstance.Delete(link);
        }

        public void LoadById(IFileDetailView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsFile link = FileServiceInstance.GetById(view.ItemId);
            ControllerUtility.AssertItemWasLoaded(link, "link", view.ItemId);

            InitializeBlank(view);
            PopulateView(link, view);
        }

        private void PopulateView(ICmsFile fromModel, IFileDetailView toView)
        {
            if (fromModel == null)
                throw new ArgumentNullException("fromModel", "fromModel is null.");
            if (toView == null)
                throw new ArgumentNullException("toView", "toView is null.");

            toView.ItemId = fromModel.Id;
            toView.DisplayOrder = fromModel.DisplayOrder;
            toView.Name = fromModel.Name;

            toView.ParentFolderId = fromModel.Parent.Id;

            toView.IsVisible = fromModel.IsVisible;

            toView.Filename = fromModel.Filename;
            toView.IsImage = fromModel.IsImage;
            toView.FileData = fromModel.Filedata.Data;
        }
    }
}
