using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public static class FolderAdapter
    {
        public static void Adapt(ICmsFolder fromFolder, IFolderInfo toFolder)
        {
            if (fromFolder == null)
                throw new ArgumentNullException("fromFolder", "fromFolder is null.");
            if (toFolder == null)
                throw new ArgumentNullException("toFolder", "toFolder is null.");
            toFolder.Id = fromFolder.Id;
            toFolder.Name = fromFolder.Name;
            toFolder.Status = fromFolder.Status;
            toFolder.Visible = fromFolder.IsVisible;
            if (fromFolder.Parent == null)
            {
                toFolder.ParentFolderId = 0;
            }
            else
            {
                toFolder.ParentFolderId = fromFolder.Parent.Id;
            }
            toFolder.DisplayOrder = fromFolder.DisplayOrder;
            toFolder.FullPath = fromFolder.FullPath;
        }

        private static void Adapt(ICollection<ICmsFolder> fromFolders, IList<IFolderInfo> toFolders)
        {
            FolderInfo toFolder;
            foreach (var fromFolder in fromFolders)
            {
                toFolder = new FolderInfo();
                Adapt(fromFolder, toFolder);
                toFolders.Add(toFolder);
            }
        }

        public static IList<IFolderInfo> Adapt(IList<ICmsFolder> fromFolders)
        {
            List<IFolderInfo> toFolders = new List<IFolderInfo>();
            Adapt(fromFolders, toFolders);

            var sorted = from temp in toFolders
                         orderby temp.FullPath
                         select temp;

            return sorted.ToList();
        }

    }
}
