﻿using System;
using Microsoft.Practices.Unity;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class FolderController : FolderItemController, IFolderController
    {
        [InjectionConstructor]
        public FolderController(IObjectFactory factory, ICmsFolderService folderService)
            : base(factory, folderService)
        {
            
        }

        public void InitializeBlank(IFolderView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            InitializeBlankFolderItemView(view);
        }

        private void PopulateView(ICmsFolder fromModel, IFolderView toView)
        {
            if (fromModel == null)
                throw new ArgumentNullException("fromModel", "fromModel is null.");
            if (toView == null)
                throw new ArgumentNullException("toView", "toView is null.");

            toView.ItemId = fromModel.Id;
            toView.DisplayOrder = fromModel.DisplayOrder;
            toView.Name = fromModel.Name;
            
            if (fromModel.Parent == null)
            {
                toView.ParentFolderId = 0;
            }
            else
            {
                toView.ParentFolderId = fromModel.Parent.Id;    
            }

            toView.IsVisible = fromModel.IsVisible;
            
            toView.Pages = WebPageAdapter.Adapt(fromModel.Pages);
            toView.Links = LinkAdapter.Adapt(fromModel.Links);
            //toView.Files = FileAdapter.Adapt(fromModel.Files);
        }

        public void LoadById(IFolderView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            if (BusinessUtility.IsItemIdSaved(view.ItemId) == false)
            {
                throw new UnknownIdException("folder", String.Empty);
            }

            var folder = FolderServiceInstance.GetById(view.ItemId);

            ControllerUtility.AssertItemWasLoaded(folder, "folder", view.ItemId);

            InitializeBlankFolderItemView(view);

            PopulateView(folder, view);
        }

        public void DeleteById(int folderId)
        {
            var folder = FolderServiceInstance.GetById(folderId);

            ControllerUtility.AssertItemWasLoaded(folder, "folder", folderId);

            FolderServiceInstance.Delete(folder);
        }

        public void Save(IFolderView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsFolder folder = null;
            ICmsFolderService facade = FolderServiceInstance;

            if (BusinessUtility.IsItemIdSaved(view.ItemId) == true)
            {
                folder = facade.GetById(view.ItemId);
                ControllerUtility.AssertItemWasLoaded(folder, "folder", view.ItemId);
            }
            else
            {
                folder = FolderServiceInstance.New(); 
                folder.Status = "ACTIVE";                     
            }

            if (BusinessUtility.IsItemIdSaved(view.ParentFolderId) == false)
            {
                folder.Parent = null;
            }
            else
            {
                var parentFolder = facade.GetById(view.ParentFolderId);

                if (parentFolder == null)
                {
                    throw new InvalidOperationException(String.Format("Parent folder with id '{0}' does not exist.", view.ParentFolderId));
                }
                else
                {
                    folder.Parent = parentFolder;
                }
            }

            folder.Name = view.Name;
            folder.DisplayOrder = view.DisplayOrder;
            folder.IsVisible = view.IsVisible;

            facade.Save(folder);
           
            view.ItemId = folder.Id;
        }
    }
}
