using System;
using Microsoft.Practices.Unity;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class LinkController : FolderItemController, ILinkController
    {
        [InjectionConstructor]
        public LinkController(ICmsFolderService folderService, ICmsLinkService linkService, IObjectFactory factory)
            : base(factory, folderService)
        {
            if (linkService == null)
            {
                throw new ArgumentNullException("linkService", "Argument cannot be null.");
            }

            m_linkServiceInstance = linkService;
        }

        private ICmsLinkService m_linkServiceInstance;
        protected ICmsLinkService LinkServiceInstance
        {
            get
            {
                return m_linkServiceInstance;
            }
        }

        public void InitializeBlank(ILinkView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            view.Url = String.Empty;

            InitializeBlankFolderItemView(view);
        }

        public void Save(ILinkView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsLink link = null;

            if (BusinessUtility.IsItemIdSaved(view.ItemId) == true)
            {
                link = LinkServiceInstance.GetById(view.ItemId);
                ControllerUtility.AssertItemWasLoaded(link, "link", view.ItemId);
            }
            else
            {
                link = ObjectFactoryInstance.Create<ICmsLink>();

                link.Status = "ACTIVE";
            }

            if (BusinessUtility.IsItemIdSaved(view.ParentFolderId) == false)
            {
                throw new ValidationException("ParentFolderId", "You must specify a valid parent folder id.");
            }
            else
            {
                var parentFolder = FolderServiceInstance.GetById(view.ParentFolderId);

                if (parentFolder == null)
                {
                    throw new UnknownIdException("folder", view.ParentFolderId);
                }
                else
                {
                    link.Parent = parentFolder;
                }
            }

            link.Name = view.Name;
            link.DisplayOrder = view.DisplayOrder;
            link.IsVisible = view.IsVisible;
            link.Url = view.Url;
            link.Target = view.Target;

            LinkServiceInstance.Save(link);

            view.ItemId = link.Id;
        }

        public void DeleteById(int linkId)
        {
            var link = LinkServiceInstance.GetById(linkId);

            ControllerUtility.AssertItemWasLoaded(link, "link", linkId);

            LinkServiceInstance.Delete(link);
        }

        public void LoadById(ILinkView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            ICmsLink link = LinkServiceInstance.GetById(view.ItemId);
            ControllerUtility.AssertItemWasLoaded(link, "link", view.ItemId);

            InitializeBlank(view);
            PopulateView(link, view);
        }

        private void PopulateView(ICmsLink fromModel, ILinkView toView)
        {
            if (fromModel == null)
                throw new ArgumentNullException("fromModel", "fromModel is null.");
            if (toView == null)
                throw new ArgumentNullException("toView", "toView is null.");

            toView.ItemId = fromModel.Id;
            toView.DisplayOrder = fromModel.DisplayOrder;
            toView.Name = fromModel.Name;

            toView.ParentFolderId = fromModel.Parent.Id;

            toView.IsVisible = fromModel.IsVisible;

            toView.Url = fromModel.Url;
            toView.Target = fromModel.Target;
        }
    }
}
