using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Admin
{
    public class MenuController : CmsControllerBase, IMenuController
    {

        public MenuController(IObjectFactory factory, ICmsFolderService folderService)
            : base(factory, folderService)
        {

        }

        private ICmsFolder FindFolderByItemKey(IList<ICmsFolder> folders, IItemKey folderItemKey)
        {
            if (folders == null)
                throw new ArgumentNullException("folders", "folders is null.");
            if (folderItemKey == null)
                throw new ArgumentNullException("folderItemKey", "folderItemKey is null.");

            var folder = (from temp in folders
                          where
                          temp.Key != null &&
                          temp.Key.ItemType == folderItemKey.ItemType &&
                          temp.Key.ItemId == folderItemKey.ItemId
                          select temp).FirstOrDefault();

            return folder;
        }

        public void Initialize(ICmsNavigationMenuView view)
        {
            if (view == null)
                throw new ArgumentNullException("view", "view is null.");

            IList<ICmsFolder> folders = FolderServiceInstance.GetList();

            var sortedAndGroupedRootFolders = (from temp in folders
                                    where temp.Parent == null && 
                                    temp.IsVisible == true
                                    orderby temp.DisplayOrder
                                    select temp).ToList();

            IList<ICmsNavigationItem> adaptedFolders = CmsNavigationItemAdapter.Adapt(sortedAndGroupedRootFolders);

            view.Folders = adaptedFolders;
            ICmsFolder selectedFolder = null;

            if (view.SelectedFolder != null)
            {
                selectedFolder = FindFolderByItemKey(folders, view.SelectedFolder);
            }
            else if (view.SelectedFolder == null && adaptedFolders.Count > 0)
            {
                selectedFolder = sortedAndGroupedRootFolders[0];                
            }

            if (selectedFolder == null)
            {
                // no data...do nothing
            }
            else
            {
                view.SelectedFolder = selectedFolder.Key;
                view.ChildItems = GetAndSortChildItems(selectedFolder);
                view.ChildFolders = GetAndSortChildFolders(selectedFolder);
            }                       
        }

        private IList<ICmsNavigationItem> GetAndSortChildFolders(ICmsFolder folder)
        {
            var sortedVisibleSubFolders = (from temp in folder.Folders
                                           where temp.IsVisible == true
                                           orderby temp.DisplayOrder
                                           select temp).ToList();

            return CmsNavigationItemAdapter.Adapt(sortedVisibleSubFolders);
        }

        private IList<ICmsNavigationItem> GetAndSortChildItems(ICmsFolder folder)
        {
            IList<ICmsNamedItem> childItems = new List<ICmsNamedItem>();

            if (folder.Links != null && folder.Links.Count > 0)
            {
                folder.Links.ForEach(x => childItems.Add(x));
            }

            if (folder.Pages != null && folder.Pages.Count > 0)
            {
                folder.Pages.ForEach(x => childItems.Add(x));
            }

            IList<ICmsNamedItem> sortedVisibleChildItems = (from temp in childItems
                                                           where temp.IsVisible == true
                                                           orderby temp.DisplayOrder
                                                           select temp).ToList();

            return CmsNavigationItemAdapter.Adapt(sortedVisibleChildItems);
        }
    }
}
