﻿using System;
using System.Linq;
using System.Collections.Generic;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.DomainModel
{
    public class CmsFolder : CmsNamedItem, ICmsFolder
    {
        public override IItemKey Key
        {
            get
            {
                return new ItemKey("folder", this.Id);
            }
        }

        private int m_ParentId;
        public int ParentId
        {
            get
            {
                if (m_Parent == null)
                {
                    return m_ParentId;
                }
                else
                {
                    return m_Parent.Id;
                }
            }
            set
            {
                if (m_Parent == null)
                {
                    m_ParentId = value;    
                }                
            }
        }

        private ICmsFolder m_Parent;
        public ICmsFolder Parent
        {
            get
            {
                return m_Parent;
            }
            set
            {
                m_Parent = value;

                if (value != null)
                {
                    value.Add(this);
                } 
            }
        }
        public string FullPath
        {
            get
            {
                if (this.Parent != null && this.Parent != this)
                {
                    return String.Format("{0}\\{1}", this.Parent.FullPath, this.Name);
                }
                else
                {
                    return this.Name;
                }
            }
        }



        private IList<ICmsFolder> m_Folders;
        public IList<ICmsFolder> Folders
        {
            get
            {
                if (m_Folders == null) m_Folders = new List<ICmsFolder>();

                return m_Folders;
            }
            set
            {
                m_Folders = value;
            }
        }




        private IList<ICmsLink> m_Links;
        public IList<ICmsLink> Links
        {
            get
            {
                if (m_Links == null) m_Links = new List<ICmsLink>();
                return m_Links;
            }
            set
            {
                m_Links = value;
            }
        }

        private IList<ICmsWebPage> m_Pages;
        public IList<ICmsWebPage> Pages
        {
            get
            {
                if (m_Pages == null) m_Pages = new List<ICmsWebPage>();
                return m_Pages;
            }
            set
            {
                m_Pages = value;
            }
        }

        private IList<ICmsFile> m_files;
        public IList<ICmsFile> Files
        {
            get
            {
                if (m_files == null) m_files = new List<ICmsFile>();
                return m_files;
            }
            set
            {
                m_files = value;
            }
        }

        public IItemKey DefaultItem
        {
            get
            {
                IItemKey returnValue = null;

                var item = FindDefaultItem();

                if (item != null)
                {
                    returnValue = item.Key;
                }

                return returnValue;
            }
        }

        public void Add(ICmsFolder child)
        {
            if (this.Folders.Contains(child) == false)
            {
                this.Folders.Add(child);
            }

            if (child.Parent != this)
            {
                child.Parent = this;
            }
        }

        public void Add(ICmsLink child)
        {
            if (this.Links.Contains(child) == false)
            {
                this.Links.Add(child);
            }

            if (child.Parent != this)
            {
                child.Parent = this;
            }
        }

        public void Add(ICmsWebPage child)
        {
            if (this.Pages.Contains(child) == false)
            {
                this.Pages.Add(child);
            }

            if (child.Parent != this)
            {
                child.Parent = this;
            }
        }

        public void Add(ICmsFile child)
        {
            if (this.Files.Contains(child) == false)
            {
                this.Files.Add(child);
            }

            if (child.Parent != this)
            {
                child.Parent = this;
            }
        }

        public ICmsNamedItem FindDefaultItem()
        {
            ICmsNamedItem returnValue = null;

            if (returnValue == null)
            {
                foreach (var link in Links)
                {
                    if (link.IsFolderDefault == true)
                    {
                        returnValue = link;
                        break;
                    }
                }
            }

            if (returnValue == null)
            {
                foreach (var page in Pages)
                {
                    if (page.IsFolderDefault == true)
                    {
                        returnValue = page;
                        break;
                    }
                }
            }

            if (returnValue == null)
            {
                if (Pages.Count > 0)
                {
                    returnValue = (from temp in Pages
                                   where temp.IsVisible == true
                                   orderby temp.DisplayOrder
                                   select temp).FirstOrDefault();
                }
            }

            if (returnValue == null)
            {
                if (Links.Count > 0)
                {
                    returnValue = (from temp in Links
                                   where temp.IsVisible == true
                                   orderby temp.DisplayOrder
                                   select temp).FirstOrDefault();
                }
            }

            return returnValue;
        }
    }
}