using System;
using System.Collections;
using System.Collections.Generic;
using Benday.Cms.Common;
using System.Text;

namespace Benday.Cms.Business.DomainModel
{
    public class LogEntry : ILogEntry
    {
        public LogEntry()
        {
        }

        private void AppendValue(StringBuilder messageBuilder, string valueName, string value)
        {
            messageBuilder.AppendFormat("{0}: {1}", valueName, value);
            messageBuilder.AppendLine();
        }

        private void AppendLine(StringBuilder messageBuilder, ILogEntryAttribute attribute)
        {
            if (attribute == null)
                return;

            AppendValue(messageBuilder, "Attribute #", attribute.Index.ToString());
            AppendValue(messageBuilder, attribute.Name, attribute.Value);
        }

        public override string ToString()
        {
            StringBuilder message = new StringBuilder();

            AppendValue(message, "LogDate", LogDate.ToString());
            AppendValue(message, "MessageType", MessageType);
            AppendValue(message, "SourceAddress", SourceAddress);
            AppendValue(message, "MessageSource", MessageSource);
            AppendValue(message, "MessageText", MessageType);
            AppendValue(message, "Username", Username);
            AppendValue(message, "SessionId", SessionId);
            AppendValue(message, "Referrer", Referrer);
            AppendValue(message, "PathInfo", PathInfo);

            if (EntryAttributes != null)
            {
                foreach (var item in EntryAttributes)
                {
                    AppendLine(message, item);
                }
            }

            return message.ToString();
        }
        
        public virtual int LogEntryId { get; set; }
        public virtual DateTime LogDate { get; set; }
        public virtual string MessageType { get; set; }
        public virtual string SourceAddress { get; set; }
        public virtual string MessageSource { get; set; }
        public virtual string MessageText { get; set; }
        public virtual string Username { get; set; }
        public virtual string SessionId { get; set; }
        public virtual string Referrer { get; set; }
        public virtual string PathInfo { get; set; }

        private IList<ILogEntryAttribute> m_attributes;
		public virtual IList<ILogEntryAttribute> EntryAttributes
        {
            get
            {
                if (m_attributes == null) m_attributes = new List<ILogEntryAttribute>();

                return m_attributes;
            }
            set
            {
                m_attributes = value;
            }
        }

        public ILogEntryAttribute AddEntryAttribute(string name, string val)
        {
            LogEntryAttribute attr = new LogEntryAttribute();

            attr.LogEntry = this;
            attr.Name = name;
            attr.Value = val;
            attr.Index = this.EntryAttributes.Count + 1;
            
            this.EntryAttributes.Add(attr);

            return attr;
        }


    }
}
