using System;
using System.Collections.Generic;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Mocks
{
    public class FolderRepositoryMock : RepositoryMock<ICmsFolder>, ICmsFolderRepository
    {
        public FolderRepositoryMock(ICmsWebPageRepository pageRepository, ICmsLinkRepository linkRepository)
        {
            if (pageRepository == null)
                throw new ArgumentNullException("pageRepository", "pageRepository is null.");
            m_pageRepositoryInstance = pageRepository;

            if (linkRepository == null)
            {
                throw new ArgumentNullException("linkRepository", "Argument cannot be null.");
            }

            m_linkRepositoryInstance = linkRepository;
        }

        private ICmsWebPageRepository m_pageRepositoryInstance;
        protected ICmsWebPageRepository PageRepositoryInstance
        {
            get { return m_pageRepositoryInstance; }
        }

        private ICmsLinkRepository m_linkRepositoryInstance;
        protected ICmsLinkRepository LinkRepositoryInstance
        {
            get { return m_linkRepositoryInstance; }
        }

        private void AddIfNotAlreadyInCollection<T>(IList<T> items, T addThis) where T : ICmsNamedItem
        {
            if (items.Contains(addThis) == false)
            {
                items.Add(addThis);
            }
        }

        private void PopulateChildWebPages(ICmsFolder item)
        {
            var results = PageRepositoryInstance.GetByFolderId(item.Id);

            if (results == null || results.Count == 0)
            {
                // do nothing
                if (item.Pages == null)
                {
                    item.Pages = new List<ICmsWebPage>();
                }
                else
                {
                    item.Pages.Clear();
                }
            }
            else
            {
                if (item.Pages == null)
                {
                    item.Pages = new List<ICmsWebPage>();
                }
                else
                {
                    item.Pages.Clear();
                }

                results.ForEach(x => AddIfNotAlreadyInCollection(item.Pages, x));
            }
        }

        private void PopulateChildLinks(ICmsFolder item)
        {
            IList<ICmsLink> results = LinkRepositoryInstance.GetByFolderId(item.Id);

            if (results == null || results.Count == 0)
            {
                // do nothing
                if (item.Links == null)
                {
                    item.Links = new List<ICmsLink>();
                }
                else
                {
                    item.Links.Clear();
                }
            }
            else
            {
                if (item.Links == null)
                {
                    item.Links = new List<ICmsLink>();
                }
                else
                {
                    item.Links.Clear();
                }

                results.ForEach(x => AddIfNotAlreadyInCollection<ICmsLink>(item.Links, x));
            }
        }

        private void PopulateChildFolders(ICmsFolder item)
        {
            var children = from temp in Items
                           where temp.Parent != null && temp.Parent.Id == item.Id
                           select temp;

            item.Folders = children.ToList();
        }

        protected override IList<ICmsFolder> BeforeReturn(IList<ICmsFolder> items)
        {
            if (items != null)
            {
                items.ForEach(x => BeforeReturn(x));
            }

            return items;
        }

        protected override ICmsFolder BeforeReturn(ICmsFolder item)
        {
            if (item != null)
            {
                PopulateChildFolders(item);
                PopulateChildWebPages(item);
                PopulateChildLinks(item);
            }

            return item;
        }
    }
}
