using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;
using System.Data.SqlClient;

using Benday.Cms.Business.DomainModel;
using Benday.Cms.Common;

namespace Benday.Cms.Business.Mocks
{
    public class RepositoryMock<T> : IRepository<T> where T : IInt32Identity
    {
        public RepositoryMock()
        {
            Items = new List<T>();
        }

        protected List<T> Items { get; set; }

        public void Reset()
        {
            Items.Clear();
        }

        public void Save(T saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (BusinessUtility.IsItemIdSaved(saveThis.Id) == false)
            {
                saveThis.Id = GetIdentityValue();
            }

            if (Items.Contains(saveThis) == false)
            {
                Items.Add(saveThis);
            }
        }

        private int m_CurrentId = 0;

        private int GetIdentityValue()
        {
            return ++m_CurrentId;
        }

        protected virtual T BeforeReturn(T item)
        {
            return item;
        }

        protected virtual IList<T> BeforeReturn(IList<T> items)
        {
            return items;
        }

        public T GetById(int id)
        {
            return BeforeReturn((from item in Items
                    where item.Id == id
                    select item).FirstOrDefault());
        }

        public IList<T> GetAll()
        {
            return BeforeReturn(Items);
        }

        public void Delete(T saveThis)
        {
            Items.Remove(saveThis);
        }
    }
}
