using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;

namespace Benday.Cms.Common
{
    [Serializable]
    public abstract class BusinessBase : IBusinessBase
    {
        public const string STATUS_ACTIVE = "ACTIVE";

        public int Id { get; set; }
        
        private string m_LastModifiedBy;

        /// <summary>
        /// Username for the person who last modified this persistent object.
        /// </summary>
        public virtual string LastModifiedBy
        {
            set
            {
                m_LastModifiedBy = value;
            }
            get
            {
                //if (string.IsNullOrEmpty(this.m_LastModifiedBy))
                //{
                //    this.m_LastModifiedBy = GetCurrentUserName();
                //}

                return m_LastModifiedBy;
            }
        }

        private string GetCurrentUserName()
        {
            return WindowsIdentity.GetCurrent().Name;
        }

        private string m_CreatedBy;
        /// <summary>
        /// Username for the person who created this persistent object.
        /// </summary>
        public virtual string CreatedBy
        {
            set
            {
                if (value != null)
                {
                    m_CreatedBy = value;
                }
            }
            get
            {
                if (string.IsNullOrEmpty(this.m_CreatedBy))
                {
                    this.m_CreatedBy = GetCurrentUserName();
                }

                return m_CreatedBy;
            }
        }


        /// <summary>
        /// When was this record created.
        /// </summary>
        public virtual DateTime CreateDate { get; set; }
        /// <summary>
        /// When was this record last modified.  This is also the property used by NHibernate 
        /// to manage datetime-based concurrency.
        /// </summary>
        public virtual DateTime LastModified { get; set; }

        private string m_Status;
        /// <summary>
        /// The status of the record. Defaults to "Active".
        /// </summary>
        public virtual string Status
        {
            get
            {
                if (string.IsNullOrEmpty(m_Status) == true)
                {
                    
                    m_Status = STATUS_ACTIVE;
                }

                return m_Status;
            }
            set
            {
                m_Status = value;
            }
        }
    }
}
