using System;
using System.Collections.Generic;

namespace Benday.Cms.Common
{
    public interface IBusinessFacade<T>
    {
        /// <summary>
        /// Get a single instance by id.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        T Get(Guid id);
        /// <summary>
        /// Get a single instance by id.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        T Get(int id);
        /// <summary>
        /// Get a single instance by property value.
        /// </summary>
        /// <param name="propertyName">Name of the object property to query</param>
        /// <param name="propertyValue">Value for the query</param>
        /// <returns></returns>
        T Get(string propertyName, object propertyValue);
        /// <summary>
        /// Get a single instance by property value.
        /// </summary>
        /// <param name="propertyName">Name of the object property to query</param>
        /// <param name="propertyValue">Value for the query</param>
        /// <param name="propertyName2">Name of the object property to query</param>
        /// <param name="propertyValue2">Value for the query</param>
        /// <returns></returns>
        T Get(string propertyName, object propertyValue, string propertyName2, object propertyValue2);
        /// <summary>
        /// Get a single object using property search values.  
        /// </summary>
        /// <param name="values">Dictionary of property search values where 
        /// the key is the name of the property and value is the value of the property.</param>
        /// <returns></returns>
        T Get(IDictionary<string, object> values);
        /// <summary>
        /// Set the name of the property to sort by and the direction of the sort (ASC or DESC).
        /// </summary>
        string SortExpression { get; set; }
        /// <summary>
        /// Get a list of all objects of type T.
        /// </summary>
        /// <returns></returns>
        IList<T> GetList();
        /// <summary>
        /// Get a list of matching instances by property value.
        /// </summary>
        /// <param name="propertyName">Name of the object property to query</param>
        /// <param name="propertyValue">Value for the query</param>
        /// <returns></returns>
        IList<T> GetList(string propertyName, object propertyValue);
        /// <summary>
        /// Get a list of all objects using property name and property value.  
        /// Property value is wrapped in a "contains" clause.
        /// </summary>
        /// <param name="propertyName"></param>
        /// <param name="propertyValue"></param>
        /// <param name="contains"></param>
        /// <returns></returns>
        IList<T> GetList(string propertyName, string propertyValue, bool contains);

        IList<T> GetList(string propertyName, string childPropertyName, int childPropertyId);

        /// <summary>
        /// Get a list of matching instances by property name/value.
        /// </summary>
        /// <param name="propertyName">Name of the object property to query</param>
        /// <param name="propertyValue">Value for the query</param>
        /// <param name="propertyName2">Name of the object property to query</param>
        /// <param name="propertyValue2">Value for the query</param>
        /// <returns></returns>
        IList<T> GetList(string propertyName, object propertyValue, string propertyName2, object propertyValue2);
        /// <summary>
        /// Get a list of objects using property search values.  
        /// </summary>
        /// <param name="values">Dictionary of property search values where 
        /// the key is the name of the property and value is the value of the property.</param>
        /// <returns></returns>
        IList<T> GetList(IDictionary<string, object> values);
        /// <summary>
        /// Save the object.
        /// </summary>
        /// <param name="saveThis"></param>
        void Save(T saveThis);
        /// <summary>
        /// Save a collection of objects.
        /// </summary>
        /// <param name="saveThese"></param>
        void Save(IList<T> saveThese);
        /// <summary>
        /// Save a collection of objects.
        /// </summary>
        /// <param name="saveThese"></param>
        void Save(IList<T> saveThese, bool withTransaction);
        /// <summary>
        /// Delete the object.
        /// </summary>
        /// <param name="deleteThis"></param>
        void Delete(T deleteThis);
        /// <summary>
        /// Delete an object by id.
        /// </summary>
        /// <param name="id"></param>
        void Delete(Guid id);
        /// <summary>
        /// Delete an object by id.
        /// </summary>
        /// <param name="id"></param>
        void Delete(int id);
    }
}
