﻿using System;
using Microsoft.Practices.Unity;
using Benday.Cms.Business.Admin;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business.Mocks;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business.Facades;
using Benday.Cms.Common;
using Benday.Cms.SqlServer;

namespace Benday.Cms.Factories
{
    public class ObjectFactory : IObjectFactory
    {
        private UnityContainer m_container;
        private UnityContainer Container
        {
            get
            {
                if (m_container == null)
                {
                    InitializeContainer("ObjectFactory");
                }

                return m_container;
            }
            set { m_container = value; }
        }

        private void InitializeContainer(string containerConfigName)
        {
            UnityContainer container = new UnityContainer();

            container.RegisterInstance<IObjectFactory>(this);

            if (containerConfigName == "UNIT_TESTS")
            {
                container.RegisterInstance<ICmsLinkRepository>(new CmsLinkRepositoryMock());
                container.RegisterInstance<ICmsFileRepository>(new CmsFileRepositoryMock());
                container.RegisterInstance<ICmsWebPageRepository>(new CmsWebPageRepositoryMock());
                container.RegisterInstance<ICmsFolderRepository>(container.Resolve<FolderRepositoryMock>());                
            }
            else
            {
                container.RegisterType<ITrafficLogger, LogFacade>();

                container.RegisterType<ICmsLinkRepository, SqlServerCmsLinkRepository>();
                container.RegisterType<ICmsFileRepository, SqlServerCmsFileRepository>();
                container.RegisterType<ICmsWebPageRepository, SqlServerCmsWebPageRepository>();
                container.RegisterType<ICmsFolderRepository, SqlServerCmsFolderRepository>();

                container.RegisterType<ILogRepository, SqlServerLogRepository>();
            }

            container.RegisterType<ICmsFolderService, CmsFolderService>();
            container.RegisterType<ICmsLinkService, CmsLinkService>();
            container.RegisterType<ICmsWebPageService, CmsWebPageService>();
            container.RegisterType<ICmsFileService, CmsFileService>();

            container.RegisterType<IFolderController, FolderController>();
            container.RegisterType<ILinkController, LinkController>();
            container.RegisterType<ICmsPageController, CmsPageController>();

            container.RegisterType<ICmsLink, CmsLink>();
            container.RegisterType<ICmsWebPage, CmsWebPage>();
            container.RegisterType<ICmsFolder, CmsFolder>();
            container.RegisterType<ICmsFile, CmsFile>();
            container.RegisterType<ICmsFileData, CmsFileData>();

            container.RegisterType<IPageDisplayController, PageDisplayController>();
            container.RegisterType<IMenuController, MenuController>();
            
            container.RegisterType<ICmsFolderToSqlServerAdapter, CmsFolderToSqlServerAdapter>();
            container.RegisterType<ICmsFileToSqlServerAdapter, CmsFileToSqlServerAdapter>();
            container.RegisterType<ICmsLinkToSqlServerAdapter, CmsLinkToSqlServerAdapter>();
            container.RegisterType<ICmsWebPageToSqlServerAdapter, CmsWebPageToSqlServerAdapter>();

            container.RegisterType<ILogEntry, LogEntry>();

            m_container = container;
        }

        public void RegisterInstance<T>(T instance)
        {
            Container.RegisterInstance<T>(instance);
        }

        public T Create<T>()
        {
            try
            {
                return Container.Resolve<T>();
            }
            catch (ResolutionFailedException ex)
            {
                throw new ObjectFactoryException(
                    String.Format("Problem creating instance of type '{0}'.", typeof(T).FullName), 
                    ex);             
            }
        }

        public void Reconfigure(string configurationName)
        {
            InitializeContainer(configurationName);
        }
    }
}
