﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using System.IO;
using System.Configuration;
using Benday.Cms.Factories;

namespace Benday.Cms.Security.WpfUi
{
    class ContentCreator
    {
        private string m_parentPath;
        public ContentCreator(string parentPath)
        {
            if (String.IsNullOrWhiteSpace(parentPath) == true)
            {
                throw new ArgumentException("Argument must be a valid string.", "parentPath");
            }
            else
            {
                if (Directory.Exists(parentPath) == false)
                {
                    throw new DirectoryNotFoundException(parentPath);
                }
                else
                {
                    m_parentPath = parentPath;
                }
            }
        }

        public void Populate()
        {
            Console.WriteLine(Folders.Count);
            Console.WriteLine(Links.Count);
            Console.WriteLine(WebPages.Count);
        }

        private ICmsFolderRepository m_folderRepositoryInstance;
        protected ICmsFolderRepository FolderRepositoryInstance
        {
            get
            {
                if (m_folderRepositoryInstance == null)
                    m_folderRepositoryInstance = ObjectFactoryInstance.Create<ICmsFolderRepository>();
                return m_folderRepositoryInstance;
            }
        }
        private ICmsFolderService m_folderServiceInstance;
        protected ICmsFolderService FolderServiceInstance
        {
            get
            {
                if (m_folderServiceInstance == null)
                    m_folderServiceInstance = ObjectFactoryInstance.Create<ICmsFolderService>();
                return m_folderServiceInstance;
            }
        }

        private ICmsWebPageRepository m_webPageRepositoryInstance;
        protected ICmsWebPageRepository WebPageRepositoryInstance
        {
            get
            {
                if (m_webPageRepositoryInstance == null)
                {
                    m_webPageRepositoryInstance = ObjectFactoryInstance.Create<ICmsWebPageRepository>();
                }

                return m_webPageRepositoryInstance;
            }
        }

        private ICmsWebPageService m_webPageServiceInstance;
        protected ICmsWebPageService WebPageServiceInstance
        {
            get
            {
                if (m_webPageServiceInstance == null)
                {
                    m_webPageServiceInstance = ObjectFactoryInstance.Create<ICmsWebPageService>();
                }

                return m_webPageServiceInstance;
            }
        }

        private IList<ICmsLink> m_links;
        protected IList<ICmsLink> Links
        {
            get
            {
                if (m_links == null)
                {
                    m_links = new List<ICmsLink>();
                    PopulateTestLinks(m_links, Folders);
                }

                return m_links;
            }
        }

        private void PopulateTestLinks(IList<ICmsLink> links, IList<ICmsFolder> folders)
        {
            links.Add(CreateLink("benday.com", "ACTIVE", true, 400, folders[0], "top", "http://www.benday.com"));
            links.Add(CreateLink("blog.benday.com", "ACTIVE", true, 500, folders[0], "top", "http://blog.benday.com"));

            links.ForEach(x => LinkServiceInstance.Save(x));
        }

        protected ICmsLink CreateLink(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent, string target, string url)
        {
            ICmsLink link = ObjectFactoryInstance.Create<ICmsLink>();
            link.Name = name;
            link.Status = status;
            link.IsVisible = isVisible;
            link.DisplayOrder = displayOrder;
            link.Target = target;
            link.Url = url;
            link.Parent = parent;
            return link;
        }

        private ICmsLinkRepository m_linkRepositoryInstance;
        protected ICmsLinkRepository LinkRepositoryInstance
        {
            get
            {
                if (m_linkRepositoryInstance == null)
                {
                    m_linkRepositoryInstance = ObjectFactoryInstance.Create<ICmsLinkRepository>();
                }

                return m_linkRepositoryInstance;
            }
        }

        private ICmsLinkService m_linkServiceInstance;
        protected ICmsLinkService LinkServiceInstance
        {
            get
            {
                if (m_linkServiceInstance == null)
                {
                    m_linkServiceInstance = ObjectFactoryInstance.Create<ICmsLinkService>();
                }

                return m_linkServiceInstance;
            }
        }


        protected ICmsFolder CreateFolder(string name, string status, bool isVisible, int displayOrder)
        {
            ICmsFolder folder = ObjectFactoryInstance.Create<ICmsFolder>();
            folder.Name = name;
            folder.Status = status;
            folder.IsVisible = isVisible;
            folder.DisplayOrder = displayOrder;
            folder.Parent = null;
            return folder;
        }

        private ICmsFolder CreateFolder(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent)
        {
            ICmsFolder folder = CreateFolder(name, status, isVisible, displayOrder);
            folder.Parent = parent;
            return folder;
        }

        private void PopulateTestFolders(IList<ICmsFolder> folders)
        {
            folders.Add(CreateFolder("Folder 1", "ACTIVE", true, 1));
            folders.Add(CreateFolder("Folder 2", "ACTIVE", true, 2));
            folders.Add(CreateFolder("Folder 3", "ACTIVE", true, 3));
            folders.ForEach(x => FolderServiceInstance.Save(x));
        }

        private IList<ICmsFolder> m_folders;
        protected IList<ICmsFolder> Folders
        {
            get
            {
                if (m_folders == null)
                {
                    m_folders = new List<ICmsFolder>();
                    PopulateTestFolders(m_folders);
                }
                return m_folders;
            }
        }

        private void PopulateTestWebPages(IList<ICmsWebPage> pages, IList<ICmsFolder> folders)
        {
            ICmsWebPage webPage = CreateWebPage("Home", "ACTIVE", true, 0, folders[0], CreateWebPageContent("SampleHomePage.txt"));
            webPage.IsFolderDefault = true;
            pages.Add(webPage);

            pages.Add(CreateWebPage("Page 0", "ACTIVE", true, 100, folders[0], CreateWebPageContent("content0", "Champagne.jpg")));
            pages.Add(CreateWebPage("Page 1", "ACTIVE", true, 200, folders[0], CreateWebPageContent("content1", "Cheeses.jpg")));
            pages.Add(CreateWebPage("Page 2", "ACTIVE", true, 2, folders[1], CreateWebPageContent("content2", "Cheeses2.jpg")));
            pages.Add(CreateWebPage("Page 3", "ACTIVE", true, 3, folders[1], CreateWebPageContent("content3", "GrowerChampagnes.jpg")));
            pages.Add(CreateWebPage("Page 4", "ACTIVE", true, 4, folders[2], CreateWebPageContent("content2", "Piano.jpg")));
            pages.Add(CreateWebPage("Page 5", "ACTIVE", true, 5, folders[2], CreateWebPageContent("content3", "WantWantPlaza.jpg")));

            pages.ForEach(x => WebPageServiceInstance.Save(x));            
        }

        public void asdf()
        {
            
        }

        private IList<ICmsWebPage> m_webPages;
        protected IList<ICmsWebPage> WebPages
        {
            get
            {
                if (m_webPages == null)
                {
                    m_webPages = new List<ICmsWebPage>();
                    PopulateTestWebPages(m_webPages, Folders);
                }

                return m_webPages;
            }
        }

        protected ICmsWebPage CreateWebPage(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent, string content)
        {
            ICmsWebPage page = ObjectFactoryInstance.Create<ICmsWebPage>();
            page.Name = name;
            page.Title = String.Format("Title: {0}", name);
            page.Status = status;
            page.IsVisible = isVisible;
            page.DisplayOrder = displayOrder;
            page.Content = content;
            page.Parent = parent;
            return page;
        }

        protected string CreateWebPageContent(string content, string filename)
        {
            return content + "<p /><img src=\"/Benday.Cms.WebUI/images/" + filename + "\" />";
        }

        protected string CreateWebPageContent(string filename)
        {
            return File.ReadAllText(Path.Combine(Environment.CurrentDirectory, filename));
        }


        internal static string GetBlobUri()
        {
            return String.Empty;
            // return new StorageAccountFactory().GetAccount().BlobEndpoint.AbsoluteUri;
        }

        public static string GetUrlToFile(int fileId)
        {
            // <Setting name="UrlToBlobFileStorage" value="{0}cmsfiledata/{1}" />
            string urlFormatString = ConfigurationManager.AppSettings["UrlToBlobFileStorage"];
            string blobUri = GetBlobUri();

            if (blobUri != null && blobUri.EndsWith("/") == false)
            {
                blobUri += "/";
            }

            if (String.IsNullOrEmpty(urlFormatString) == true)
            {
                throw new InvalidOperationException("Configuration value 'UrlToBlobFileStorage' is not specified.");
            }

            return String.Format(urlFormatString, blobUri, fileId);
        }

        protected static void EnsureNoDefaultItem(ICmsFolder folderToTestWith)
        {
            foreach (var item in folderToTestWith.Pages)
                item.IsFolderDefault = false;
            foreach (var item in folderToTestWith.Links)
                item.IsFolderDefault = false;
        }
        private byte[] ReadFileAsByteArray(string parentPath, string testFilename)
        {
            string path = Path.Combine(parentPath, testFilename);
            
            return File.ReadAllBytes(path);
        }
        
        private IObjectFactory m_objectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory();                    
                }

                return m_objectFactoryInstance;
            }
        }
    }
}
