﻿using System;
using System.Linq;
using System.Windows;
using System.Web.Security;
using Benday.Cms.Common;
using Benday.Cms.Factories;

namespace Benday.Cms.Security.WpfUi
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void m_btnRefreshUsers_Click(object sender, RoutedEventArgs e)
        {
            EnsureRolesExist();

            int totalRecords = 0;

            var users = Membership.GetAllUsers(0, 0x7fffffff, out totalRecords);
            
            var usersForDataBinding = MembershipUserWrapper.Wrap(users);
            
            m_grid.DataContext = usersForDataBinding;

            CompareStrings();
        }

        private void CompareStrings()
        {
            var str1 = "+P@$$w0rd:1:FrT2cwHP0EuWHnia4r6CBQ==::iU8NbxPs25AS1qLyU3WSTFq2EQlUMjZ1s7GhOuuqLyI=";
            var str2 = "+P@$$w0rd:1:FrT2cwHP0EuWHnia4r6CBQ==::GpHkPm0qE4Jes8KGa2oyBJhb8Qc=";

            for (int i = 0; i < str2.Length; i++)
            {
                Console.Write(str1[i]);
                Console.Write(":");
                Console.Write(str2[i]);
                Console.Write("  --  ");
                if (str1[i] == str2[i])
                {
                    Console.WriteLine("same");
                }
                else
                {
                    Console.WriteLine("not same");
                }
            }
        }

        private void EnsureRolesExist()
        {
            var allRoles = Roles.GetAllRoles();

            if (allRoles == null || allRoles.Length == 0 || allRoles.Contains(MembershipUserWrapper.ROLE_NAME_ADMINISTRATOR) == false)
            {
                Roles.CreateRole(MembershipUserWrapper.ROLE_NAME_ADMINISTRATOR);
            }
        }

        private void m_btnAddAdministrator_Click(object sender, RoutedEventArgs e)
        {
            var username = "administrator";
            var password = "P@$$w0rd";
            var user = Membership.CreateUser(username, password, "user@user.com");
            Roles.AddUserToRole(username, MembershipUserWrapper.ROLE_NAME_ADMINISTRATOR);

            MessageBox.Show(
                String.Format(
                "Created an administrator in the CMS database.{2}Username: {0}{2}Password: {1}", 
                username, password, Environment.NewLine));
        }

        private void m_btnDeleteAllUsers_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show("You are about to delete all the users in your database.  Are you sure?", "Are you sure?", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
            {
                DeleteAllUsers();
            }
        }

        private void DeleteAllUsers()
        {
            int totalRecords = 0;

            var users = Membership.GetAllUsers(0, 0x7fffffff, out totalRecords);

            foreach (MembershipUser user in users)
            {
                Membership.DeleteUser(user.UserName);
            }
        }

        private void m_btnCreateTestContent_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                CreateTestContent();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory();
                }

                return m_objectFactoryInstance;
            }
        }

        private ICmsFolderService m_folderServiceInstance;
        public ICmsFolderService FolderServiceInstance
        {
            get 
            {
                if (m_folderServiceInstance == null)
                {
                    m_folderServiceInstance = ObjectFactoryInstance.Create<ICmsFolderService>();
                }

                return m_folderServiceInstance; 
            }
        }        

        private void CreateTestContent()
        {
            ContentCreator creator = new ContentCreator(Environment.CurrentDirectory);
            creator.Populate();
        }

        private void m_btnDeleteAllContent_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show("You are about to delete all the content in your database.  Are you sure?", "Are you sure?", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
            {
                new ContentDeletor().DeleteAllContent();
            }
        }
    }
}
