using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;


namespace Benday.Cms.SqlServer.IntegrationTests
{

    [TestClass]
    public class SqlServerCmsFolderAdapterFixture : UnitTestBase
    {
        public SqlServerCmsFolderAdapterFixture()
        {
        }

        [TestMethod]
        public void SqlServerCmsFolderAdapter_AdaptICmsFolderToCmsFolderRow()
        {
            ICmsFolder fromValue = ObjectFactoryInstance.Create<ICmsFolder>();

            fromValue.CreateDate = DateTime.Now.AddDays(-10);
            fromValue.CreatedBy = "ben";
            fromValue.DisplayOrder = 1234;
            fromValue.Id = 52525;
            fromValue.IsVisible = true;
            fromValue.LastModified = DateTime.Now.AddDays(10);
            fromValue.LastModifiedBy = "test";
            fromValue.Name = "test name";
            fromValue.Status = "test status";

            CmsDataset.CmsFolderRow toValue = new CmsDataset().CmsFolder.NewCmsFolderRow();

            AdapterInstance.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue, true);
        }

        private ICmsFolderToSqlServerAdapter m_AdapterInstance;
        public ICmsFolderToSqlServerAdapter AdapterInstance
        {
            get
            {
                if (m_AdapterInstance == null)
                {
                    m_AdapterInstance = 
                        ObjectFactoryInstance.Create<ICmsFolderToSqlServerAdapter>();
                }

                return m_AdapterInstance;
            }
        }        

        [TestMethod]
        public void SqlServerCmsFolderAdapter_AdaptCmsFolderRowToICmsFolder()
        {
            CmsDataset.CmsFolderRow fromValue = new CmsDataset().CmsFolder.NewCmsFolderRow();

            fromValue.CreateDate = DateTime.Now.AddDays(-10);
            fromValue.CreatedBy = "ben";
            fromValue.Id = 10;
            fromValue.DisplayOrder = 1234;
            fromValue.Id = 52525;
            fromValue.IsVisible = true;
            fromValue.LastModified = DateTime.Now.AddDays(10);
            fromValue.LastModifiedBy = "test";
            fromValue.Name = "test name";
            fromValue.Status = "test status";
            fromValue.ParentFolderId = 4321;

            ICmsFolder toValue = ObjectFactoryInstance.Create<ICmsFolder>();

            AdapterInstance.Adapt(fromValue, toValue);

            AssertAreEqual(fromValue, toValue);
        }

        private void AssertAreEqual(ICmsFolder expected, CmsDataset.CmsFolderRow actual, bool ignoreId)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");

            if (ignoreId == false)
            {
                Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
        }

        private void AssertAreEqual(ICmsFolder expected, CmsDataset.CmsFolderRow actual)
        {
            AssertAreEqual(expected, actual, false);
        }

        private void AssertAreEqual(CmsDataset.CmsFolderRow expected, ICmsFolder actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
        }
    }
}
