using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;


namespace Benday.Cms.SqlServer.IntegrationTests
{

    [TestClass]
    public class SqlServerCmsLinkAdapterFixture : UnitTestBase
    {
        public SqlServerCmsLinkAdapterFixture()
        {
        }

        [TestMethod]
        public void SqlServerCmsLinkAdapter_ICmsLinkToCmsLinkRow()
        {
            ICmsLink domainType = ObjectFactoryInstance.Create<ICmsLink>();

            domainType.CreateDate = DateTime.Now.AddDays(-10);
            domainType.CreatedBy = "ben";
            domainType.DisplayOrder = 1234;
            domainType.Id = 52525;
            domainType.IsVisible = true;
            domainType.LastModified = DateTime.Now.AddDays(10);
            domainType.LastModifiedBy = "test";
            domainType.Name = "test name";
            domainType.Status = "test status";
            domainType.ParentId = 543210;

            domainType.IsFolderDefault = true;
            domainType.Target = "_top";
            domainType.Url = "http://www.benday.com";

            CmsDataset.CmsLinkRow storageType = new CmsDataset().CmsLink.NewCmsLinkRow();
            
            AdapterInstance.Adapt(domainType, storageType);

            AssertAreEqual(domainType, storageType);
        }

        private ICmsLinkToSqlServerAdapter m_AdapterInstance;
        public ICmsLinkToSqlServerAdapter AdapterInstance
        {
            get
            {
                if (m_AdapterInstance == null)
                {
                    m_AdapterInstance =
                        ObjectFactoryInstance.Create<ICmsLinkToSqlServerAdapter>();
                }

                return m_AdapterInstance;
            }
        }

        private void AssertAreEqual(ICmsLink expected, CmsDataset.CmsLinkRow actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");

            Assert.AreEqual<string>(expected.Target, actual.Target, "Target");
            Assert.AreEqual<bool>(expected.IsFolderDefault, actual.IsFolderDefault, "IsFolderDefault");
            Assert.AreEqual<string>(expected.Url, actual.Url, "Url");
        }

        [TestMethod]
        public void SqlServerCmsLinkAdapter_AdaptCmsLinkRowToICmsLink()
        {
            CmsDataset.CmsLinkRow storageType = new CmsDataset().CmsLink.NewCmsLinkRow();

            storageType.CreateDate = DateTime.Now.AddDays(-10);
            storageType.CreatedBy = "ben";
            storageType.Id = 10;
            storageType.DisplayOrder = 1234;
            storageType.Id = 52525;
            storageType.IsVisible = true;
            storageType.LastModified = DateTime.Now.AddDays(10);
            storageType.LastModifiedBy = "test";
            storageType.Name = "test name";
            storageType.Status = "test status";

            storageType.IsFolderDefault = true;
            storageType.Target = "_top";
            storageType.Url = "http://www.benday.com";

            ICmsLink domainType = ObjectFactoryInstance.Create<ICmsLink>();

            AdapterInstance.Adapt(
                storageType, 
                domainType);

            AssertAreEqual(storageType, domainType);
        }

        private void AssertAreEqual(CmsDataset.CmsLinkRow expected, ICmsLink actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");

            Assert.AreEqual<string>(expected.Target, actual.Target, "Target");
            Assert.AreEqual<bool>(expected.IsFolderDefault, actual.IsFolderDefault, "IsFolderDefault");
            Assert.AreEqual<string>(expected.Url, actual.Url, "Url");
        }
    }
}
