using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;


namespace Benday.Cms.SqlServer.IntegrationTests
{
    [TestClass]
    public class SqlServerCmsLinkRepositoryFixture : SqlServerRepositoryFixtureBase
    {
        public SqlServerCmsLinkRepositoryFixture()
        {

        }

        [TestMethod]
        public void SqlServerCmsLinkRepository_SaveLink()
        {
            ICmsLink link = CreateLink(Folders[1], "test1");

            LinkRepositoryInstance.Save(link);

            AssertIsSaved(link.Id, "link.Id");
        }

        [TestMethod]
        public void SqlServerCmsLinkRepository_GetByParentFolderId()
        {
            Assert.AreNotEqual<int>(0, Links.Count, "Link count was 0.");

            IList<ICmsLink> links;

            links = LinkRepositoryInstance.GetByFolderId(Folders[0].Id);
            Assert.IsNotNull(links);
            Assert.AreEqual<int>(0, links.Count, "Link count should be 0 for parent folder id '{0}'.", Folders[0].Id);

            links = LinkRepositoryInstance.GetByFolderId(Folders[1].Id);
            Assert.IsNotNull(links);
            Assert.AreEqual<int>(2, links.Count, "Link count should be 2.");
            Assert.AreEqual<int>(Folders[1].Id, links[0].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[1].Id, links[1].ParentId, "Parent didn't match.");

            links = LinkRepositoryInstance.GetByFolderId(Folders[2].Id);
            Assert.IsNotNull(links);
            Assert.AreEqual<int>(3, links.Count, "Link count should be 3.");
            Assert.AreEqual<int>(Folders[2].Id, links[0].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[2].Id, links[1].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[2].Id, links[2].ParentId, "Parent didn't match.");
        }


        [TestMethod]
        public void SqlServerCmsLinkRepository_GetAllLinks()
        {
            Assert.AreNotEqual<int>(0, Links.Count, "Link count was 0.");

            var links = LinkRepositoryInstance.GetAll();

            Assert.IsNotNull(links, "Links collection was null.");
            Assert.AreEqual<int>(Links.Count, links.Count, "Link count was wrong.");
        }

        [TestMethod]
        public void SqlServerCmsLinkRepository_GetLinkById()
        {

            Assert.AreNotEqual<int>(0, Links.Count, "Link count was 0.");

            var expected = Links[1];

            Assert.AreNotEqual<int>(0, expected.ParentId, "ParentId was 0 (not saved).");

            AssertIsSaved(expected, "Link");


            var actual = LinkRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");

            Assert.AreNotEqual<int>(0, actual.ParentId, "Should have a ParentId.");
            Assert.IsNotNull(actual.Parent, "Reloaded parent is null.");
            Assert.AreEqual<int>(expected.ParentId, actual.Parent.Id, "Parent Id did not match.");
        }
    }
}
