using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Benday.Cms.SqlServer.IntegrationTests
{
    [TestClass]
    public class SqlServerCmsWebPageAdapterFixture : UnitTestBase
    {
        public SqlServerCmsWebPageAdapterFixture()
        {
        }

        [TestMethod]
        public void SqlServerCmsWebPageAdapter_ICmsWebPageToCmsWebPageRow()
        {
            ICmsWebPage domainType = ObjectFactoryInstance.Create<ICmsWebPage>();

            domainType.CreateDate = DateTime.Now.AddDays(-10);
            domainType.CreatedBy = "ben";
            domainType.DisplayOrder = 1234;
            domainType.Id = 52525;
            domainType.IsVisible = true;
            domainType.LastModified = DateTime.Now.AddDays(10);
            domainType.LastModifiedBy = "test";
            domainType.Name = "test name";
            domainType.Status = "test status";

            domainType.IsFolderDefault = true;
            domainType.Title = "title";

            CmsDataset.CmsWebPageRow storageType = new CmsDataset().CmsWebPage.NewCmsWebPageRow();
            AdapterInstance.Adapt(domainType, storageType);

            AssertAreEqual(domainType, storageType, true);
        }

        private ICmsWebPageToSqlServerAdapter m_AdapterInstance;
        public ICmsWebPageToSqlServerAdapter AdapterInstance
        {
            get
            {
                if (m_AdapterInstance == null)
                {
                    m_AdapterInstance =
                        ObjectFactoryInstance.Create<ICmsWebPageToSqlServerAdapter>();
                }

                return m_AdapterInstance;
            }
        }

        private void AssertAreEqual(ICmsWebPage expected, CmsDataset.CmsWebPageRow actual)
        {
            AssertAreEqual(expected, actual, false);
        }

        private void AssertAreEqual(ICmsWebPage expected, CmsDataset.CmsWebPageRow actual, bool ignoreId)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");

            if (ignoreId == false)
            {
                Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");

            Assert.AreEqual<bool>(expected.IsFolderDefault, actual.IsFolderDefault, "IsFolderDefault");
            Assert.AreEqual<string>(expected.Title, actual.Title, "Title");
        }

        [TestMethod]
        public void SqlServerCmsWebPageAdapter_AdaptCmsWebPageRowToICmsWebPage()
        {
            CmsDataset.CmsWebPageRow storageType = new CmsDataset().CmsWebPage.NewCmsWebPageRow();

            storageType.CreateDate = DateTime.Now.AddDays(-10);
            storageType.CreatedBy = "ben";
            storageType.Id = 10;
            storageType.DisplayOrder = 1234;
            storageType.Id = 52525;
            storageType.IsVisible = true;
            storageType.LastModified = DateTime.Now.AddDays(10);
            storageType.LastModifiedBy = "test";
            storageType.Name = "test name";
            storageType.Status = "test status";

            storageType.IsFolderDefault = true;
            storageType.Title = "title";

            ICmsWebPage domainType = ObjectFactoryInstance.Create<ICmsWebPage>();
            AdapterInstance.Adapt(storageType, 
                domainType);

            AssertAreEqual(storageType, domainType);
        }

        private void AssertAreEqual(CmsDataset.CmsWebPageRow expected, ICmsWebPage actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");

            Assert.AreEqual<string>(expected.Title, actual.Title, "Title");
            Assert.AreEqual<bool>(expected.IsFolderDefault, actual.IsFolderDefault, "IsFolderDefault");
        }
    }
}
