using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Threading;

namespace Benday.Cms.SqlServer.IntegrationTests
{

    [TestClass]
    public class SqlServerCmsWebPageRepositoryFixture : SqlServerRepositoryFixtureBase
    {
        [TestMethod]
        public void SqlServerCmsWebPageRepository_Save()
        {
            ICmsWebPage page = CreateWebPage(Folders[1], "test1", true);

            WebPageRepositoryInstance.Save(page);

            AssertIsSaved(page, "page");
        }

        [TestMethod]
        public void SqlServerCmsWebPageRepository_GetByParentFolderId()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "WebPage count was 0.");

            IList<ICmsWebPage> pages;

            pages = WebPageRepositoryInstance.GetByFolderId(Folders[0].Id);
            Assert.IsNotNull(pages);
            Assert.AreEqual<int>(0, pages.Count, "WebPage count should be 0.");

            pages = WebPageRepositoryInstance.GetByFolderId(Folders[1].Id);
            Assert.IsNotNull(pages);
            Assert.AreEqual<int>(2, pages.Count, "WebPage count should be 2.");
            Assert.AreEqual<int>(Folders[1].Id, pages[0].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[1].Id, pages[1].ParentId, "Parent didn't match.");

            pages = WebPageRepositoryInstance.GetByFolderId(Folders[2].Id);
            Assert.IsNotNull(pages);
            Assert.AreEqual<int>(3, pages.Count, "WebPage count should be 3.");
            Assert.AreEqual<int>(Folders[2].Id, pages[0].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[2].Id, pages[1].ParentId, "Parent didn't match.");
            Assert.AreEqual<int>(Folders[2].Id, pages[2].ParentId, "Parent didn't match.");
        }


        [TestMethod]
        public void SqlServerCmsWebPageRepository_GetAll()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "Link count was 0.");

            var pages = WebPageRepositoryInstance.GetAll();

            Assert.IsNotNull(pages, "WebPages collection was null.");
            Assert.AreEqual<int>(WebPages.Count, pages.Count, "WebPage count was wrong.");

            foreach (var page in pages)
            {
                Assert.IsNotNull(page.Content, "Content was null.");
                Assert.IsNotNull(page.Content, "Content was null.");
                Assert.AreNotEqual<string>(String.Empty, page.Content, "Content should not be empty.");
            }
        }

        [TestMethod]
        public void SqlServerCmsWebPageRepository_GetById()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "WebPage count was 0.");

            var expected = WebPages[1];

            Assert.AreNotEqual<string>(String.Empty, 
                expected.Content, 
                "Content should not be empty at the start of the test.");

            AssertIsSaved(expected, "expected");

            var actual = WebPageRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");

            Assert.IsNotNull(actual.Content, "Content was null.");
            Assert.AreNotEqual<string>(String.Empty, actual.Content, "Content should not be empty.");

            Assert.IsNotNull(actual.Parent, "Reloaded parent is null.");
            Assert.AreEqual<int>(expected.ParentId, actual.Parent.Id, "Parent Id did not match.");
        }


        [TestMethod]
        public void SqlServerCmsWebPageRepository_ModifyContent()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "WebPage count was 0.");

            var expected = WebPages[1];

            AssertIsSaved(expected, "expected");

            var actual = WebPageRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");

            Assert.IsNotNull(actual.Content, "Content was null.");
            Assert.IsNotNull(actual.Content, "Content was null.");
            Assert.AreNotEqual<string>(String.Empty, actual.Content, "Content should not be empty.");

            string originalContent = actual.Content;

            // modify the content
            actual.Content = Guid.NewGuid().ToString();

            WebPageRepositoryInstance.Save(actual);

            var reloaded = WebPageRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(reloaded, "Reloaded was null.");

            Assert.IsNotNull(reloaded.Content, "Content was null.");
            Assert.IsNotNull(reloaded.Content, "Content was null.");
            Assert.AreEqual<string>(actual.Content, reloaded.Content, "Content did not match after reloaded.");
        }

        [TestMethod]
        public void SqlServerCmsWebPageRepository_ChangeParentFolder()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "WebPage count was 0.");

            var expected = WebPages[1];

            int originalFolderId = expected.ParentId;

            AssertIsSaved(expected, "expected");

            var actual = WebPageRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");

            Assert.IsNotNull(actual.Content, "Content was null.");
            Assert.IsNotNull(actual.Content, "Content was null.");
            Assert.AreNotEqual<string>(String.Empty, actual.Content, "Content should not be empty.");

            var newFolder = Folders[2];

            Assert.AreNotEqual<int>(newFolder.Id, actual.ParentId, "New folder should not be the same as the old folder.");

            int newFolderId = newFolder.Id;

            Assert.AreNotEqual<int>(originalFolderId, newFolderId, "Folder ids should be different.");

            actual.Parent = newFolder;

            WebPageRepositoryInstance.Save(actual);

            var reloaded = WebPageRepositoryInstance.GetById(expected.Id);

            Assert.IsNotNull(reloaded, "Reloaded was null.");

            Assert.AreEqual<int>(newFolder.Id, reloaded.ParentId, "Reloaded parent folder id was wrong.");
            Assert.AreEqual<int>(newFolder.Id, reloaded.Parent.Id, "Reloaded parent object folder id was wrong.");
            Assert.AreNotEqual<int>(originalFolderId, reloaded.Parent.Id, "Reloaded parent object folder id didn't change.");
        }
    }
}
