using System;
using Benday.Cms.Common;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Benday.Cms.SqlServer.IntegrationTests
{
    [TestClass]
    public class SqlServerLogRepositoryFixture : SqlServerRepositoryFixtureBase
    {
        [TestMethod]
        public void LogRepository_WriteToLog()
        {
            var startingLogEntryCount = ExecuteSqlQuery("SELECT * FROM LogEntry");
            Assert.AreEqual<int>(0, startingLogEntryCount.Rows.Count, "Row count should be 0.");
            var instance = ObjectFactoryInstance.Create<ILogRepository>();
            ILogEntry entry = CreateLogEntry();
            instance.Save(entry);

            var endingLogEntryCount = ExecuteSqlQuery("SELECT * FROM LogEntry");
            Assert.AreNotEqual<int>(0, endingLogEntryCount.Rows.Count, "Row count should be not 0.");
        }

        private ILogEntry CreateLogEntry()
        {
            var entry = ObjectFactoryInstance.Create<ILogEntry>();
            entry.LogDate = DateTime.UtcNow;
            entry.MessageSource = UnitTestUtility.GetReasonablyUniqueString("source_");
            entry.MessageText = UnitTestUtility.GetReasonablyUniqueString("text_");
            entry.MessageType = UnitTestUtility.GetReasonablyUniqueString("type_");
            entry.PathInfo = UnitTestUtility.GetReasonablyUniqueString("pathinfo_");
            entry.Referrer = UnitTestUtility.GetReasonablyUniqueString("referrer_");
            entry.SessionId = UnitTestUtility.GetReasonablyUniqueString("SessionId_");
            entry.SourceAddress = UnitTestUtility.GetReasonablyUniqueString("SourceAddress_");
            entry.Username = UnitTestUtility.GetReasonablyUniqueString("username_");
            return entry;
        }
        [TestMethod]
        public void LogRepository_CreateInstance()
        {
            var instance = ObjectFactoryInstance.Create<ILogRepository>();
            Assert.IsNotNull(instance, "Instance was null.");
        }
    }
}
