﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using Benday.Cms.Business;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Diagnostics;
using Benday.Cms.Factories;

namespace Benday.Cms.SqlServer.IntegrationTests
{
    [TestClass]
    public abstract class UnitTestBase
    {
        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory();
                    m_objectFactoryInstance.Reconfigure(ObjectFactoryConfigurationName);
                    RegisterAdditionalInstancesInObjectFactory(m_objectFactoryInstance);
                }

                return m_objectFactoryInstance;
            }
        }

        protected virtual string ObjectFactoryConfigurationName
        {
            get { return "SQL_INTEGRATION_TESTS"; }
        }


        public TestContext TestContext { get; set; }

        protected virtual void RegisterAdditionalInstancesInObjectFactory(IObjectFactory factory)
        {

        }

        protected virtual void OnTestInitialize()
        {

        }

        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }

        [TestCleanup]
        public void TestCleanup()
        {
            OnTestCleanup();
        }

        protected virtual void OnTestCleanup()
        {
            m_objectFactoryInstance = null;
        }

        protected void AssertDateValueIsWithinLastSecondUtc(DateTime actual, string valueName)
        {
            DateTime now = DateTime.UtcNow;
            var span = now - actual;
            Assert.IsTrue(span.TotalMilliseconds < 1000, "Value for '{0}' was older than 1 second.  Total milliseconds was '{1}'.", valueName, span.TotalMilliseconds);
        }
        protected static void AssertDateTimeFieldsAreDefault(ICmsFolder root)
        {
            Assert.AreEqual<DateTime>(default(DateTime), root.CreateDate, "CreateDate should be default value.");
            Assert.AreEqual<DateTime>(default(DateTime), root.LastModified, "LastModified should be default value.");
        }
        protected void AssertDateTimeFieldsWereUpdatedOnCreate(ICmsFolder root)
        {
            AssertDateValueIsWithinLastSecondUtc(root.CreateDate, "folder.CreateDate");
            AssertDateValueIsWithinLastSecondUtc(root.LastModified, "folder.LastModified");
        }
    }
}
