using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Benday.Cms.SqlServer.IntegrationTests
{
    public static class UnitTestUtility
    {
        public static int GetReasonablyUniqueInt(string prefix)
        {
            var guidAsByteArray = Guid.NewGuid().ToByteArray();
            int total = 0;
            foreach (byte byteFromGuid in guidAsByteArray)
                total += ((int)byteFromGuid);
            return prefix.Length + total;
        }
        public static DateTime GetReasonablyUniqueDateTime(string prefix)
        {
            return DateTime.Now.AddHours(Convert.ToDouble(GetReasonablyUniqueInt(prefix)));
        }
        public static string GetReasonablyUniqueString(string prefix)
        {
            string fullValue = String.Format("{0}{1}", prefix, Guid.NewGuid());
            if (fullValue.Length > 20)
                return fullValue.Substring(0, 20);
            else
                return fullValue;
        }
        
        public static IList<string> GetList(params string[] args)
        {
            var returnValue = new List<string>();
            if (args == null || args.Length == 0)
                return returnValue;
            else
            {
                foreach (var arg in args)
                    returnValue.Add(arg);
                return returnValue;
            }
        }
        public static void AssertIsStringEmpty(string actual, string message)
        {
            Assert.AreEqual<string>(String.Empty, actual, "'{0}' should be String.Empty.", message);
        }
        public static void AssertIsNotNullOrStringEmpty(string actual, string message)
        {
            if (String.IsNullOrWhiteSpace(actual) == true)
                Assert.Fail("'{0}' should not be null or whitespace or empty string.", message);
        }
        
        
        
        public static void AssertAreEqual(IList<string> expected, IList<string> actual, string message)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");
            if (String.IsNullOrEmpty(message))
                throw new ArgumentException("message is null or empty.", "message");
            var expectedSorted = expected.OrderBy(x => x).ToList();
            var actualSorted = actual.OrderBy(x => x).ToList();
            Assert.AreEqual<int>(expectedSorted.Count(), actualSorted.Count(), "Item count didn't match.");
            for (int i = 0; i < expectedSorted.Count(); i++)
                Assert.AreEqual<string>(expectedSorted[i], actualSorted[i], "Value at index '{0}' did not match for '{1}'", i, message);
        }
        
    }
}
