using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class CmsFolderToSqlServerAdapter : ICmsFolderToSqlServerAdapter
    {
        public CmsFolderToSqlServerAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public void Adapt(Benday.Cms.SqlServer.CmsDataset.CmsFolderDataTable fromValues, IList<ICmsFolder> toValues)
        {
            if (fromValues == null)
                throw new ArgumentNullException("fromValues", "fromValues is null.");
            if (toValues == null)
                throw new ArgumentNullException("toValues", "toValues is null.");

            toValues.Clear();

            if (fromValues.Count > 0)
            {
                ICmsFolder toValue;

                foreach (var fromValue in fromValues)
                {
                    toValue = ObjectFactoryInstance.Create<ICmsFolder>();

                    Adapt(fromValue, toValue);

                    toValues.Add(toValue);
                }
            }
        }

        public void Adapt(ICmsFolder fromValue, CmsDataset.CmsFolderRow toValue)
        {
            if (fromValue == null)
            {
                throw new ArgumentNullException("fromValue", "Argument cannot be null.");
            }

            if (toValue == null)
            {
                throw new ArgumentNullException("toValue", "Argument cannot be null.");
            }

            // Id is read-only
            // toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;

            if (fromValue.ParentId == 0)
            {
                toValue.SetParentFolderIdNull();
            }
            else
            {
                toValue.ParentFolderId = fromValue.ParentId;    
            }
            
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;
        }

        public void Adapt(CmsDataset.CmsFolderRow fromValue, ICmsFolder toValue)
        {
            if (fromValue == null)
            {
                throw new ArgumentNullException("fromValue", "Argument cannot be null.");
            }

            if (toValue == null)
            {
                throw new ArgumentNullException("toValue", "Argument cannot be null.");
            }

            toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;

            if (fromValue.IsParentFolderIdNull() == true)
            {
                toValue.ParentId = 0;
            }
            else
            {
                toValue.ParentId = fromValue.ParentFolderId;    
            }
            
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;
        }
    }
}
