using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class CmsLinkToSqlServerAdapter : ICmsLinkToSqlServerAdapter
    {
        public CmsLinkToSqlServerAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }
        
        public void Adapt(Benday.Cms.SqlServer.CmsDataset.CmsLinkRow fromValue, ICmsLink toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.Target = fromValue.Target;
            toValue.Url = fromValue.Url;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;

            if (fromValue.IsParentFolderIdNull() == true)
            {
                toValue.ParentId = 0;
            }
            else
            {
                toValue.ParentId = fromValue.ParentFolderId;
            }
            
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;
        }
        public void Adapt(CmsDataset.CmsLinkRow fromValue, ICmsFolder toValue)
        {
            throw new NotImplementedException();
        }
        public void Adapt(CmsDataset.CmsLinkDataTable fromValues, List<ICmsLink> toValues)
        {
            if (fromValues == null)
                throw new ArgumentNullException("fromValues", "fromValues is null.");
            if (toValues == null)
                throw new ArgumentNullException("toValues", "toValues is null.");

            toValues.Clear();

            if (fromValues.Count > 0)
            {
                ICmsLink toValue;

                foreach (var fromValue in fromValues)
                {
                    toValue = ObjectFactoryInstance.Create<ICmsLink>();

                    Adapt(fromValue, toValue);

                    toValues.Add(toValue);
                }
            }
        }
        public void Adapt(ICmsLink fromValue, Benday.Cms.SqlServer.CmsDataset.CmsLinkRow toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.Target = fromValue.Target;
            toValue.Url = fromValue.Url;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;

            if (fromValue.ParentId == 0)
            {
                toValue.SetParentFolderIdNull();
            }
            else
            {
                toValue.ParentFolderId = fromValue.ParentId;
            }
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;
        }
    }
}
