using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class CmsWebPageToSqlServerAdapter : ICmsWebPageToSqlServerAdapter
    {
        public CmsWebPageToSqlServerAdapter(IObjectFactory factory)
        {
            if (factory == null)
                throw new ArgumentNullException("factory", "factory is null.");

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public void Adapt(Benday.Cms.SqlServer.CmsDataset.CmsWebPageDataTable fromValues, List<ICmsWebPage> toValues)
        {
            if (fromValues == null)
                throw new ArgumentNullException("fromValues", "fromValues is null.");
            if (toValues == null)
                throw new ArgumentNullException("toValues", "toValues is null.");

            toValues.Clear();

            if (fromValues.Count > 0)
            {
                ICmsWebPage toValue;

                foreach (var fromValue in fromValues)
                {
                    toValue = ObjectFactoryInstance.Create<ICmsWebPage>();

                    Adapt(fromValue, toValue);

                    toValues.Add(toValue);
                }
            }
        }

        public void Adapt(CmsDataset.CmsWebPageRow fromValue, ICmsWebPage toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.Title = fromValue.Title;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;
            if (fromValue.IsParentFolderIdNull() == true)
            {
                toValue.ParentId = 0;
            }
            else
            {
                toValue.ParentId = fromValue.ParentFolderId;
            }
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;

            if (fromValue.IsHtmlNull() == true)
            {
                toValue.Content = String.Empty;
            }
            else
            {
                toValue.Content = fromValue.Html;    
            }            
        }

        public void Adapt(ICmsWebPage fromValue, CmsDataset.CmsWebPageRow toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            // Id is read-only
            // toValue.Id = fromValue.Id;
            toValue.LastModified = fromValue.LastModified;
            toValue.Name = fromValue.Name;
            toValue.Title = fromValue.Title;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;
            if (fromValue.ParentId == 0)
            {
                toValue.SetParentFolderIdNull();
            }
            else
            {
                toValue.ParentFolderId = fromValue.ParentId;
            }
            toValue.Status = fromValue.Status;
            toValue.CreateDate = fromValue.CreateDate;
            toValue.CreatedBy = fromValue.CreatedBy;
            toValue.LastModifiedBy = fromValue.LastModifiedBy;
            toValue.Html = fromValue.Content;
        }
    }
}
