using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class SqlServerCmsFileRepository : 
        SqlServerCmsRepositoryBase<ICmsFile, CmsDatasetTableAdapters.CmsFileTableAdapter>, 
        ICmsFileRepository
    {
        private const string FRIENDLY_OBJECT_NAME = "file";

        public SqlServerCmsFileRepository(
            IObjectFactory factory,
            ICmsFileToSqlServerAdapter adapter)
            : base(factory)
        {
            if (adapter == null)
            {
                throw new ArgumentNullException("adapter", "Argument cannot be null.");
            }

            m_ObjectAdapterInstance = adapter;
        }

        private ICmsFileToSqlServerAdapter m_ObjectAdapterInstance;
        private ICmsFileToSqlServerAdapter ObjectAdapterInstance
        {
            get { return m_ObjectAdapterInstance; }
        }

        public IList<ICmsFile> GetByFolderId(int folderId)
        {
            throw new NotImplementedException();
        }

        public override IList<ICmsFile> GetAll()
        {
            var fromValues = TableAdapterInstance.GetData();

            var toValues = new List<ICmsFile>();

            ObjectAdapterInstance.Adapt(fromValues, toValues);

            return toValues;
        }

        public override ICmsFile GetById(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                return null;
            }
            else
            {
                var fromValue = fromValues[0];

                var toValue =
                    ObjectFactoryInstance.Create<ICmsFile>();

                ObjectAdapterInstance.Adapt(fromValue, toValue);

                return toValue;
            }
        }

        public override void Save(ICmsFile saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (saveThis.Id == 0)
            {
                Insert(saveThis);
            }
            else
            {
                Update(saveThis);
            }
        }

        private void Insert(ICmsFile insertThis)
        {
            CmsDataset dataset = new CmsDataset();

            var toValue = dataset.CmsFile.NewCmsFileRow();

            ObjectAdapterInstance.Adapt(insertThis, toValue);

            dataset.CmsFile.AddCmsFileRow(toValue);

            TableAdapterInstance.Update(toValue);

            // refresh the object with any changed data 
            // (eg. @@IDENTITY, timestamps)
            ObjectAdapterInstance.Adapt(toValue, insertThis);
        }

        private void Update(ICmsFile updateThis)
        {
            if (updateThis == null)
            {
                throw new ArgumentNullException("updateThis", "Argument cannot be null.");
            }

            var toValues = TableAdapterInstance.GetDataById(updateThis.Id);

            if (toValues == null || toValues.Count == 0)
            {
                throw new UnknownIdException(FRIENDLY_OBJECT_NAME, updateThis.Id);
            }
            else
            {
                var toValue = toValues[0];

                ObjectAdapterInstance.Adapt(updateThis, toValue);

                TableAdapterInstance.Update(toValue);

                // refresh the object with any changed data 
                // (eg. @@IDENTITY, timestamps)
                ObjectAdapterInstance.Adapt(toValue, updateThis);
            }
        }

        private void Delete(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                // no nothing
            }
            else
            {
                fromValues[0].Delete();

                TableAdapterInstance.Update(fromValues);
            }
        }

        public override void Delete(ICmsFile deleteThis)
        {
            if (deleteThis == null)
            {
                throw new ArgumentNullException("deleteThis", "Argument cannot be null.");
            }

            Delete(deleteThis.Id);
        }
    }
}
