using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;

namespace Benday.Cms.SqlServer
{
    public class SqlServerCmsLinkRepository :
        SqlServerCmsRepositoryBase<ICmsLink, CmsDatasetTableAdapters.CmsLinkTableAdapter>, 
        ICmsLinkRepository
    {
        private const string FRIENDLY_OBJECT_NAME = "link";
        public SqlServerCmsLinkRepository(
            IObjectFactory factory,
            ICmsLinkToSqlServerAdapter adapter,
            ICmsFolderRepository folderRepository)
            : base(factory)
        {
            if (folderRepository == null)
                throw new ArgumentNullException("folderRepository", "folderRepository is null.");
            
            if (adapter == null)
            {
                throw new ArgumentNullException("adapter", "Argument cannot be null.");
            }

            m_ObjectAdapterInstance = adapter;
            m_FolderRepository = folderRepository;
        }

        private ICmsFolderRepository m_FolderRepository;
        private ICmsFolderRepository FolderRepository
        {
            get { return m_FolderRepository; }
        }

        private ICmsLinkToSqlServerAdapter m_ObjectAdapterInstance;
        private ICmsLinkToSqlServerAdapter ObjectAdapterInstance
        {
            get { return m_ObjectAdapterInstance; }
        }

        public IList<ICmsLink> GetByFolderId(int folderId)
        {
            var fromValues = TableAdapterInstance.GetDataByParentFolderId(folderId);

            var toValues = new List<ICmsLink>();

            ObjectAdapterInstance.Adapt(fromValues, toValues);

            UpdateParentFolderLazyLoader(toValues, FolderRepository);

            Cleanup();

            return toValues;            
        }

        public override IList<ICmsLink> GetAll()
        {
            var fromValues = TableAdapterInstance.GetData();

            var toValues = new List<ICmsLink>();

            ObjectAdapterInstance.Adapt(fromValues, toValues);

            UpdateParentFolderLazyLoader(toValues, FolderRepository);

            Cleanup();

            return toValues;
        }

        public override ICmsLink GetById(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                throw new UnknownIdException("link", id);
            }
            else
            {
                var fromValue = fromValues[0];

                var toValue =
                    ObjectFactoryInstance.Create<ICmsLink>();

                ObjectAdapterInstance.Adapt(fromValue, toValue);

                UpdateParentFolderLazyLoader(toValue, FolderRepository);

                Cleanup();

                return toValue;
            }
        }

        public override void Save(ICmsLink saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (saveThis.Id == 0)
            {
                Insert(saveThis);
            }
            else
            {
                Update(saveThis);
            }

            Cleanup();
        }

        private void Insert(ICmsLink insertThis)
        {
            CmsDataset dataset = new CmsDataset();

            var toValue = dataset.CmsLink.NewCmsLinkRow();

            UpdateDataForInsert(insertThis as IBusinessBase);

            ObjectAdapterInstance.Adapt(insertThis, toValue);

            dataset.CmsLink.AddCmsLinkRow(toValue);

            TableAdapterInstance.Update(toValue);

            // refresh the object with any changed data 
            // (eg. @@IDENTITY, timestamps)
            ObjectAdapterInstance.Adapt(toValue, insertThis);
        }

        private void Update(ICmsLink updateThis)
        {
            if (updateThis == null)
            {
                throw new ArgumentNullException("updateThis", "Argument cannot be null.");
            }

            UpdateDataForUpdate(updateThis as IBusinessBase);

            var toValues = TableAdapterInstance.GetDataById(updateThis.Id);

            if (toValues == null || toValues.Count == 0)
            {
                throw new UnknownIdException(FRIENDLY_OBJECT_NAME, updateThis.Id);
            }
            else
            {
                var toValue = toValues[0];

                ObjectAdapterInstance.Adapt(updateThis, toValue);

                TableAdapterInstance.Update(toValue);

                // refresh the object with any changed data 
                // (eg. @@IDENTITY, timestamps)
                ObjectAdapterInstance.Adapt(toValue, updateThis);
            }
        }

        private void Delete(int id)
        {
            var fromValues = TableAdapterInstance.GetDataById(id);

            if (fromValues == null || fromValues.Count == 0)
            {
                // no nothing
            }
            else
            {
                fromValues[0].Delete();

                TableAdapterInstance.Update(fromValues);
            }
        }

        public override void Delete(ICmsLink deleteThis)
        {
            if (deleteThis == null)
            {
                throw new ArgumentNullException("deleteThis", "Argument cannot be null.");
            }

            Delete(deleteThis.Id);

            Cleanup();
        }
    }
}
