using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Benday.Cms.Common;
using System.Data.SqlClient;
using System.Configuration;
using System.Security.Principal;

namespace Benday.Cms.SqlServer
{
    public abstract class SqlServerCmsRepositoryBase<TDomain, TTableAdapter> :
        SqlServerRepositoryBase<TTableAdapter>, IRepository<TDomain> 
        where TDomain : IBusinessBase
        where TTableAdapter : IRepositoryTableAdapter, new()
    {
        public SqlServerCmsRepositoryBase(IObjectFactory factory)
        {
            if (factory == null)
            {
                throw new ArgumentNullException("factory", "Argument cannot be null.");
            }

            m_ObjectFactoryInstance = factory;
        }

        private IObjectFactory m_ObjectFactoryInstance;
        protected IObjectFactory ObjectFactoryInstance
        {
            get { return m_ObjectFactoryInstance; }
        }

        public abstract IList<TDomain> GetAll();        
        public abstract TDomain GetById(int id);        
        public abstract void Save(TDomain saveThis);
        public abstract void Delete(TDomain deleteThis);
        
        protected void UpdateDataForUpdate(IBusinessBase updateThis)
        {
            if (updateThis != null)
            {
                updateThis.LastModified = DateTime.UtcNow;
                updateThis.LastModifiedBy = WindowsIdentity.GetCurrent().Name;
            }
        }

        protected void UpdateDataForInsert(IBusinessBase updateThis)
        {
            if (updateThis != null)
            {
                updateThis.CreateDate = DateTime.UtcNow;
                updateThis.CreatedBy = WindowsIdentity.GetCurrent().Name;

                UpdateDataForUpdate(updateThis);
            }
        }

        protected void UpdateParentFolderLazyLoader(ICmsFolderParented toValue, ICmsFolderRepository repository)
        {
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");
            if (repository == null)
                throw new ArgumentNullException("repository", "repository is null.");

            if (toValue.ParentId > 0)
            {
                toValue.Parent = new CmsFolderLazyLoadProxy(repository, toValue.ParentId);
            }
        }

        protected void UpdateParentFolderLazyLoader(IEnumerable<ICmsFolderParented> toValues, ICmsFolderRepository repository)
        {
            if (toValues == null)
                return;
            else
            {
                if (repository == null)
                    throw new ArgumentNullException("repository", "Argument cannot be null.");
                foreach (var toValue in toValues)
                {
                    UpdateParentFolderLazyLoader(toValue, repository);
                }
            }
        }
    }
}
