using System;
using System.Configuration;
using System.Data.SqlClient;

namespace Benday.Cms.SqlServer
{
    public class SqlServerRepositoryBase<TTableAdapter> 
        where TTableAdapter : IRepositoryTableAdapter, new()
    {
        public SqlServerRepositoryBase()
        {
        }
        
        private string m_ConnectionString;
        protected string ConnectionString
        {
            get
            {
                if (m_ConnectionString == null)
                {
                    var connection = ConfigurationManager.ConnectionStrings["(default)"];
                    if (connection == null)
                        throw new InvalidOperationException("There is no connection string named '(default)' in the current configuration.");
                    else
                        m_ConnectionString = connection.ConnectionString;
                }
                return m_ConnectionString;
            }
        }

        private SqlConnection m_ConnectionInstance;
        protected SqlConnection ConnectionInstance
        {
            get
            {
                if (m_ConnectionInstance == null)
                {
                    m_ConnectionInstance = new SqlConnection(ConnectionString);
                    m_ConnectionInstance.Open();
                }
                return m_ConnectionInstance;
            }
        }

        private TTableAdapter m_TableAdapterInstance;
        protected TTableAdapter TableAdapterInstance
        {
            get
            {
                if (m_TableAdapterInstance == null)
                {
                    m_TableAdapterInstance = new TTableAdapter();

                    m_TableAdapterInstance.Connection = ConnectionInstance;
                }

                return m_TableAdapterInstance;
            }
        }

        private bool IgnoreCleanup()
        {
            string value = ConfigurationManager.AppSettings["DataAccess.Repositories.IgnoreCleanup"];
            
            if (value != null && value == "true")
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        protected void Cleanup()
        {
            bool ignoreCleanup = IgnoreCleanup();

            if (ignoreCleanup == false)
            {
                if (m_TableAdapterInstance != null)
                {
                    m_TableAdapterInstance.Dispose();
                    m_TableAdapterInstance = default(TTableAdapter);
                }

                if (m_ConnectionInstance != null)
                {
                    m_ConnectionInstance.Close();
                    m_ConnectionInstance.Dispose();
                    m_ConnectionInstance = null;
                }
            }            
        }
    }
}
