using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using Benday.Cms.Business.Admin;
using Benday.Cms.Business;
using Rhino.Mocks;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;
using Benday.Cms.UnitTests.UserInterfaceTests;
using System.IO;

namespace Benday.Cms.UnitTests
{
    public abstract class CmsFixtureBase : UnitTestBase
    {
        public CmsFixtureBase()
        {

        }

        private MockRepository m_mockRepositoryInstance;
        public MockRepository MockRepositoryInstance
        {
            get
            {
                if (m_mockRepositoryInstance == null)
                {
                    m_mockRepositoryInstance = new MockRepository();
                }

                return m_mockRepositoryInstance;
            }
        }

        private ICmsFolderRepository m_folderRepositoryInstance;
        public ICmsFolderRepository FolderRepositoryInstance
        {
            get
            {
                if (m_folderRepositoryInstance == null)
                    m_folderRepositoryInstance = ObjectFactoryInstance.Create<ICmsFolderRepository>();
                return m_folderRepositoryInstance;
            }
        }
        private ICmsFolderService m_folderServiceInstance;
        public ICmsFolderService FolderServiceInstance
        {
            get
            {
                if (m_folderServiceInstance == null)
                    m_folderServiceInstance = ObjectFactoryInstance.Create<ICmsFolderService>(); 
                return m_folderServiceInstance;
            }
        }

        private ICmsWebPageRepository m_webPageRepositoryInstance;
        public ICmsWebPageRepository WebPageRepositoryInstance
        {
            get
            {
                if (m_webPageRepositoryInstance == null)
                {
                    m_webPageRepositoryInstance = ObjectFactoryInstance.Create<ICmsWebPageRepository>();
                }

                return m_webPageRepositoryInstance;
            }            
        }

        private ICmsWebPageService m_webPageServiceInstance;
        public ICmsWebPageService WebPageServiceInstance
        {
            get
            {
                if (m_webPageServiceInstance == null)
                {
                    m_webPageServiceInstance = ObjectFactoryInstance.Create<ICmsWebPageService>(); 
                }

                return m_webPageServiceInstance;
            }            
        }

        private IList<ICmsLink> m_links;
        public IList<ICmsLink> Links
        {
            get
            {
                if (m_links == null)
                {
                    m_links = new List<ICmsLink>();
                    PopulateTestLinks(m_links, Folders);
                }

                return m_links;
            }
        }

        private void PopulateTestLinks(IList<ICmsLink> links, IList<ICmsFolder> folders)
        {
            links.Add(CreateLink("link0", "ACTIVE", true, 0, folders[0], "top", "url0"));
            links.Add(CreateLink("link1", "ACTIVE", true, 1, folders[0], "top", "url1"));
            links.Add(CreateLink("link2", "ACTIVE", true, 2, folders[1], "top", "url2"));
            links.Add(CreateLink("link3", "ACTIVE", true, 3, folders[1], "top", "url3"));
            links.Add(CreateLink("link4", "ACTIVE", true, 4, folders[2], "top", "url4"));
            links.Add(CreateLink("link5", "ACTIVE", true, 2, folders[2], "top", "url5"));
            links.Add(CreateLink("link6", "ACTIVE", true, 10, folders[2], "top", "url6"));
            links.Add(CreateLink("link7", "ACTIVE", true, 1, folders[2], "top", "url7"));
            links.Add(CreateLink("link8", "ACTIVE", true, 6, folders[2], "top", "url7"));

            links.ForEach(x => LinkServiceInstance.Save(x));
            links.ForEach(x => Assert.AreNotEqual<int>(default(int), x.Id, "Item wasn't saved."));
        }

        protected ICmsLink CreateLink(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent, string target, string url)
        {
            ICmsLink link = ObjectFactoryInstance.Create<ICmsLink>();
            link.Name = name;
            link.Status = status;
            link.IsVisible = isVisible;
            link.DisplayOrder = displayOrder;
            link.Target = target;
            link.Url = url;
            link.Parent = parent;
            return link;
        }

        private ICmsLinkRepository m_linkRepositoryInstance;
        public ICmsLinkRepository LinkRepositoryInstance
        {
            get
            {
                if (m_linkRepositoryInstance == null)
                {
                    m_linkRepositoryInstance = ObjectFactoryInstance.Create<ICmsLinkRepository>();
                }

                return m_linkRepositoryInstance;
            }
        }

        private ICmsLinkService m_linkServiceInstance;
        public ICmsLinkService LinkServiceInstance
        {
            get
            {
                if (m_linkServiceInstance == null)
                {
                    m_linkServiceInstance = ObjectFactoryInstance.Create<ICmsLinkService>(); 
                }

                return m_linkServiceInstance;
            }
        }

        
        protected ICmsFolder CreateFolder(string name, string status, bool isVisible, int displayOrder)
        {
            ICmsFolder folder = ObjectFactoryInstance.Create<ICmsFolder>();
            folder.Name = name;
            folder.Status = status;
            folder.IsVisible = isVisible;
            folder.DisplayOrder = displayOrder;
            folder.Parent = null;
            return folder;
        }

        private ICmsFolder CreateFolder(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent)
        {
            ICmsFolder folder = CreateFolder(name, status, isVisible, displayOrder);
            folder.Parent = parent;
            return folder;
        }

        private void PopulateTestFolders(IList<ICmsFolder> folders)
        {
            folders.Add(CreateFolder("name0", "ACTIVE", true, 1));
            folders.Add(CreateFolder("name1", "INACTIVE", false, 2));
            folders.Add(CreateFolder("name2", "ACTIVE", false, 3, folders[0]));
            folders.Add(CreateFolder("name3", "INACTIVE", true, 4, folders[0]));
            folders.Add(CreateFolder("name4", "INACTIVE", true, 5, folders[1]));
            folders.Add(CreateFolder("name5", "INACTIVE", true, 7, folders[1]));
            folders.Add(CreateFolder("name6", "INACTIVE", true, 6, folders[1]));
            folders.Add(CreateFolder("name7", "INACTIVE", true, 8, folders[1]));
            folders.Add(CreateFolder("name8", "INACTIVE", true, 2, folders[1]));
            folders.ForEach(x => FolderServiceInstance.Save(x));
            folders.ForEach(x => Assert.AreNotEqual<int>(default(int), x.Id, "Item wasn't saved."));
        }

        private IList<ICmsFolder> m_folders;
        public IList<ICmsFolder> Folders
        {
            get
            {
                if (m_folders == null)
                {
                    m_folders = new List<ICmsFolder>();
                    PopulateTestFolders(m_folders);
                }
                return m_folders;
            }
        }

        private void PopulateTestWebPages(IList<ICmsWebPage> pages, IList<ICmsFolder> folders)
        {
            pages.Add(CreateWebPage("page0", "ACTIVE", true, 0, folders[0], CreateWebPageContent("content0")));
            pages.Add(CreateWebPage("page1", "ACTIVE", true, 1, folders[0], CreateWebPageContent("content1")));
            pages.Add(CreateWebPage("page2", "ACTIVE", true, 2, folders[1], CreateWebPageContent("content2")));
            pages.Add(CreateWebPage("page3", "ACTIVE", true, 3, folders[1], CreateWebPageContent("content3")));

            pages.ForEach(x => WebPageServiceInstance.Save(x));
            pages.ForEach(x => Assert.AreNotEqual<int>(default(int), x.Id, "Item wasn't saved."));
        }

        private IList<ICmsWebPage> m_webPages;
        public IList<ICmsWebPage> WebPages
        {
            get
            {
                if (m_webPages == null)
                {
                    m_webPages = new List<ICmsWebPage>();
                    PopulateTestWebPages(m_webPages, Folders);
                }

            	return m_webPages;
            }
        }

        protected void AssertAreEqual(ICmsFolder expected, ICmsFolder actual)
        {
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
            if (expected.Parent == null)
            {
                Assert.IsNull(actual.Parent, "Parent folder should be null.");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id,
                                    actual.Parent.Id,
                                    "ParentFolder.Id");
            }
        }
        protected ICmsWebPage CreateWebPage(string name, string status, bool isVisible, int displayOrder, ICmsFolder parent, string content)
        {
            ICmsWebPage page = ObjectFactoryInstance.Create<ICmsWebPage>();
            page.Name = name;
            page.Status = status;
            page.IsVisible = isVisible;
            page.DisplayOrder = displayOrder;
            page.Content = content;
            page.Parent = parent;
            return page;
        }

        protected string CreateWebPageContent(string content)
        {
            return content;
        }

        protected void AssertAreEqual(ICmsWebPage expected, ICmsWebPage actual)
        {
            Assert.AreEqual<string>(expected.Title, actual.Title, "Title");
            Assert.AreEqual<string>(expected.Content, actual.Content, "Content");
            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
            if (expected.Parent == null)
            {
                Assert.IsNull(actual.Parent, "Parent folder should be null.");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.Parent.Id, "ParentFolder.Id");
            }
        }

        protected void AssertAreEqual(ICmsLink expected, ICmsLink actual)
        {
            Assert.AreEqual<string>(expected.Target, actual.Target, "Target");
            Assert.AreEqual<string>(expected.Url, actual.Url, "Url");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<DateTime>(expected.CreateDate, actual.CreateDate, "CreateDate");
            Assert.AreEqual<string>(expected.CreatedBy, actual.CreatedBy, "CreatedBy");
            Assert.AreEqual<int>(expected.DisplayOrder, actual.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "IsVisible");
            Assert.AreEqual<DateTime>(expected.LastModified, actual.LastModified, "LastModified");
            Assert.AreEqual<string>(expected.LastModifiedBy, actual.LastModifiedBy, "LastModifiedBy");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");
            if (expected.Parent == null)
            {
                Assert.IsNull(actual.Parent, "Parent folder should be null.");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id,
                                    actual.Parent.Id,
                                    "ParentFolder.Id");
            }
        }

        protected void AssertAreEqual(IItemKey expected, IItemKey actual)
        {
            AssertAreEqual(expected, actual, "Item key does not match");
        }

        protected void AssertAreEqual(IItemKey expected, IItemKey actual, string errorMessagePrefix)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");
            Assert.AreEqual<int>(expected.ItemId, actual.ItemId, "{0}: ItemId", errorMessagePrefix);
            Assert.AreEqual<string>(expected.ItemType, actual.ItemType, "{0}: ItemType", errorMessagePrefix);
        }
        protected static void EnsureNoDefaultItem(ICmsFolder folderToTestWith)
        {
            foreach (var item in folderToTestWith.Pages)
                item.IsFolderDefault = false;
            foreach (var item in folderToTestWith.Links)
                item.IsFolderDefault = false;
        }

        /*
        private IList<ICmsFile> m_files;
        private ICmsFileService m_fileServiceInstance;
        private void PopulateTestFiles(IList<ICmsFile> files, IList<ICmsFolder> folders)
        {
            ICmsFile file;
            file = ObjectFactoryInstance.Create<ICmsFile>();
            file.DisplayOrder = 100;
            file.IsImage = true;
            file.IsVisible = true;
            file.Name = "MVPLogo_Small.jpg";
            file.Parent = folders[0];
            file.Status = "ACTIVE";
            file.Filedata.Data = ReadFileAsByteArray("MVPLogo_Small.jpg");
            folders[0].Add(file);
            files.Add(file);
            file = ObjectFactoryInstance.Create<ICmsFile>();
            file.DisplayOrder = 200;
            file.IsImage = true;
            file.IsVisible = false;
            file.Name = "siteLogo.gif";
            file.Parent = folders[0];
            file.Status = "ACTIVE";
            file.Filedata.Data = ReadFileAsByteArray("siteLogo.gif");
            folders[0].Add(file);
            files.Add(file);
            file = ObjectFactoryInstance.Create<ICmsFile>();
            file.DisplayOrder = 100;
            file.IsImage = true;
            file.IsVisible = true;
            file.Name = "MVPLogo_Small2.jpg";
            file.Parent = folders[1];
            file.Status = "ACTIVE";
            file.Filedata.Data = ReadFileAsByteArray("MVPLogo_Small.jpg");
            folders[1].Add(file);
            files.Add(file);
        }
        private byte[] ReadFileAsByteArray(string testFilename)
        {
            string path = Path.Combine(TestContext.TestDeploymentDir, testFilename);
            Assert.IsTrue(File.Exists(path), "Could not find test filename '{0}' at '{1}'.", testFilename, path);
            return File.ReadAllBytes(path);
        }
        public IList<ICmsFile> Files
        {
            get
            {
                if (m_files == null)
                {
                    m_files = new List<ICmsFile>();
                    PopulateTestFiles(m_files, Folders);
                    m_files.ForEach(x => FileServiceInstance.Save(x));
                }
                return m_files;
            }
        }
        public ICmsFileService FileServiceInstance
        {
            get
            {
                if (m_fileServiceInstance == null)
                    m_fileServiceInstance = ObjectFactoryInstance.Create<ICmsFileService>();
                return m_fileServiceInstance;
            }
        }
        */
    }
}
