using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.DomainModel;
using System.Security.Principal;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests
{
    [TestClass]
    public class CmsLinkServiceFixture : CmsFixtureBase
    {
        public CmsLinkServiceFixture()
        {
        }

        [TestMethod]
        public void CmsLink_GetAll()
        {
            Assert.AreNotEqual<int>(0, Links.Count, "Page count should not be 0 at the start of the test.");

            var result = LinkServiceInstance.GetList();

            Assert.IsNotNull(result, "Result was null.");
            Assert.AreEqual<int>(Links.Count, result.Count, "Result count was wrong.");
        }

        [TestMethod]
        public void CmsLink_GetById()
        {
            int count = Links.Count;
            ICmsLink expected;
            ICmsLink actual;

            for (int i = count - 1; i >= 0; i--)
            {
                expected = Links[i];

                actual = LinkServiceInstance.GetById(expected.Id);

                Assert.IsNotNull(actual, "Actual was null.");
                AssertAreEqual(expected, actual);
            }
        }        

        [TestMethod]
        public void CmsLink_SaveToMockRepositoryUpdatesConcurrencyColumns()
        {
            string expectedUsername = WindowsIdentity.GetCurrent().Name;

            ICmsLink folder = CreateLink("name", "ACTIVE", true, 1, null, String.Empty, String.Empty);

            Assert.AreEqual<int>(default(int), folder.Id, "Id should be String.Empty");
            Assert.AreEqual<DateTime>(DateTime.MinValue, folder.CreateDate, "CreatedDate was wrong.");
            Assert.AreEqual<DateTime>(DateTime.MinValue, folder.LastModified, "LastModified was wrong.");

            LinkServiceInstance.Save(folder);

            Assert.AreNotEqual<int>(default(int), folder.Id, "Id should not be String.Empty after save.");
            Assert.AreEqual<string>(expectedUsername, folder.CreatedBy, "CreatedBy was wrong.");
            Assert.AreEqual<string>(expectedUsername, folder.LastModifiedBy, "LastModifiedBy was wrong.");
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, folder.CreateDate, "CreatedDate was not updated during the save.");
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, folder.LastModified, "LastModified was not updated during the save.");
        }

        [TestMethod]
        public void CmsLink_Delete()
        {
            int count = Links.Count;
            ICmsLink expected = Links[0];
            ICmsLink actual = LinkServiceInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");

            LinkServiceInstance.Delete(actual);

            actual = LinkServiceInstance.GetById(expected.Id);
            Assert.IsNull(actual, "Actual should be null after delete.");
        }
    }
}
