using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Security.Principal;
using System.Linq;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests
{
    [TestClass]
    public class CmsWebPageServiceFixture : CmsFixtureBase
    {
        public CmsWebPageServiceFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(Folders);
            Assert.IsNotNull(WebPages);
            Assert.IsNotNull(Links);
        }

        [TestMethod]
        public void CmsWebPage_GetAll()
        {
            Assert.AreNotEqual<int>(0, WebPages.Count, "Page count should not be 0 at the start of the test.");

            var result = WebPageServiceInstance.GetList();

            Assert.IsNotNull(result, "Result was null.");
            Assert.AreEqual<int>(WebPages.Count, result.Count, "Result count was wrong.");
        }

        [TestMethod]
        public void CmsWebPage_GetById()
        {
            int count = WebPages.Count;
            ICmsWebPage expected;
            ICmsWebPage actual;

            for (int i = count - 1; i >= 0; i--)
            {
                expected = WebPages[i];

                actual = WebPageServiceInstance.GetById(expected.Id);

                Assert.IsNotNull(actual, "Actual was null.");
                AssertAreEqual(expected, actual);
            }
        }

        [TestMethod]
        public void CmsWebPage_SaveToMockRepositoryUpdatesConcurrencyColumns()
        {
            string expectedUsername = WindowsIdentity.GetCurrent().Name;

            ICmsWebPage page = CreateWebPage("name", "ACTIVE", true, 1, null, CreateWebPageContent("content"));

            Assert.AreEqual<int>(default(int), page.Id, "Id should be String.Empty");
            Assert.AreEqual<DateTime>(DateTime.MinValue, page.CreateDate, "CreatedDate was wrong.");
            Assert.AreEqual<DateTime>(DateTime.MinValue, page.LastModified, "LastModified was wrong.");

            WebPageServiceInstance.Save(page);

            Assert.AreNotEqual<int>(default(int), page.Id, "Id should not be String.Empty after save.");
            Assert.AreEqual<string>(expectedUsername, page.CreatedBy, "CreatedBy was wrong.");
            Assert.AreEqual<string>(expectedUsername, page.LastModifiedBy, "LastModifiedBy was wrong.");
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, page.CreateDate, "CreatedDate was not updated during the save.");
            Assert.AreNotEqual<DateTime>(DateTime.MinValue, page.LastModified, "LastModified was not updated during the save.");
        }

        [TestMethod]
        public void CmsWebPage_SetIsFolderDefaultOnPageUpdatesFolderDefaultOnParent()
        {
            var folderToTestWith = (from temp in FolderServiceInstance.GetList()
                                    where temp.Pages != null && temp.Pages.Count >= 2
                                    select temp).FirstOrDefault();

            Assert.IsNotNull(folderToTestWith, "folderToTestWith was null -- eg. couldn't find a folder with at least 2 items in it.");

            EnsureNoDefaultItem(folderToTestWith);

            var pageToTestWith = folderToTestWith.Pages[1];

            Assert.IsFalse(pageToTestWith.IsFolderDefault, "Page should not be folder default yet.");

            pageToTestWith.IsFolderDefault = true;

            Assert.IsNotNull(folderToTestWith.DefaultItem, "There should be a default item now.");
            AssertAreEqual(folderToTestWith.DefaultItem, pageToTestWith.Key);
        }

        [TestMethod]
        public void CmsWebPage_Delete()
        {
            int count = WebPages.Count;
            ICmsWebPage expected = WebPages[0];
            ICmsWebPage actual = WebPageServiceInstance.GetById(expected.Id);

            Assert.IsNotNull(actual, "Actual was null.");

            WebPageServiceInstance.Delete(actual);

            actual = WebPageServiceInstance.GetById(expected.Id);
            Assert.IsNull(actual, "Actual should be null after delete.");
        }
    }
}
