using System;

using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business;
using Benday.Cms.Common;
using Benday.Cms.Factories;

namespace Benday.Cms.UnitTests
{
    [TestClass]
    public class UnitTestBase 
    {
        private IObjectFactory m_objectFactoryInstance;
        public IObjectFactory ObjectFactoryInstance
        {
            get
            {
                if (m_objectFactoryInstance == null)
                {
                    m_objectFactoryInstance = new ObjectFactory();
                    m_objectFactoryInstance.Reconfigure(ObjectFactoryConfigurationName);
                    RegisterAdditionalInstancesInObjectFactory(m_objectFactoryInstance);
                }

                return m_objectFactoryInstance;
            }
        }

        protected virtual string ObjectFactoryConfigurationName
        {
            get { return "UNIT_TESTS"; }
        }
        

        public TestContext TestContext { get; set; }

        protected virtual void RegisterAdditionalInstancesInObjectFactory(IObjectFactory factory)
        {
            
        }

        protected virtual void OnTestInitialize()
        {
            
        }

        [TestInitialize]
        public void TestInitialize()
        {
            OnTestInitialize();
        }
    }
}
