using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.UnitTests;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    public abstract class AdminUserInterfaceFixture : CmsFixtureBase
    {
        public AdminUserInterfaceFixture()
        {
        }

        private IFolderListView m_folderListViewInstance;
        public IFolderListView FolderListViewInstance
        {
            get
            {
                if (m_folderListViewInstance == null)
                    m_folderListViewInstance = MockRepositoryInstance.Stub<IFolderListView>();
                return m_folderListViewInstance;
            }
        }
        private IFolderController m_folderControllerInstance;
        public IFolderController FolderControllerInstance
        {
            get
            {
                if (m_folderControllerInstance == null)
                {
                    m_folderControllerInstance = ObjectFactoryInstance.Create<IFolderController>();
                }
                return m_folderControllerInstance;
            }
        }

        private ICmsPageController m_pageControllerInstance;
        public ICmsPageController PageControllerInstance
        {
            get
            {
                if (m_pageControllerInstance == null)
                {
                    m_pageControllerInstance = ObjectFactoryInstance.Create<ICmsPageController>();
                }
                return m_pageControllerInstance;
            }
        }

        private IPageDisplayController m_pageDisplayControllerInstance;
        public IPageDisplayController PageDisplayControllerInstance
        {
            get
            {
                if (m_pageDisplayControllerInstance == null)
                {
                    m_pageDisplayControllerInstance =
                        ObjectFactoryInstance.Create<IPageDisplayController>();
                }
                return m_pageDisplayControllerInstance;
            }
        }

        private ILinkController m_linkControllerInstance;
        public ILinkController LinkControllerInstance
        {
            get
            {
                if (m_linkControllerInstance == null)
                {
                    m_linkControllerInstance = ObjectFactoryInstance.Create<ILinkController>();
                }
                return m_linkControllerInstance;
            }
        }

        protected IList<ICmsFolder> GetNonRootFolders()
        {
            return (from temp in Folders where temp.Parent != null select temp).ToList();
        }

        protected IList<ICmsFolder> GetRootFolders()
        {
            return (from temp in Folders where temp.Parent == null select temp).ToList();
        }

        private IList<ICmsWebPage> m_pages;
        public IList<ICmsWebPage> Pages
        {
            get
            {
                if (m_pages == null)
                {
                    m_pages = new List<ICmsWebPage>();
                    m_pages = CreateWebPagesAndReturnAll(GetRootFolders());
                }
                return m_pages;
            }
        }

        private IList<ICmsWebPage> CreateWebPagesAndReturnAll(IList<ICmsFolder> folders)
        {
            IList<ICmsWebPage> pages = new List<ICmsWebPage>();

            ICmsFolder folder0;
            ICmsFolder folder1;
            ICmsFolder subfolder0;

            Assert.IsTrue(folders.Count > 1, "Should be at least 2 folders at the beginning of the test.");

            folder0 = folders[0];
            folder1 = folders[1];

            Assert.IsTrue(folder0.Folders.Count > 0, "Folder0 should have at least one subfolder.");
            subfolder0 = folder0.Folders[0];

            var pageF0_P2 = WebPageServiceInstance.New("PageF0_P2", folder0, true, 200);
            pageF0_P2.IsFolderDefault = true;

            var pageF0_P1 = WebPageServiceInstance.New("PageF0_P1", folder0, false, 100);

            var pageF0_F1_P1 = WebPageServiceInstance.New("PageF0_F1_P1", subfolder0, false, 300);
            var pageF0_F1_P2 = WebPageServiceInstance.New("PageF0_F1_P2", subfolder0, false, 100);
            var pageF0_F1_P3 = WebPageServiceInstance.New("PageF0_F1_P3", subfolder0, false, 200);

            var pageF1_P2 = WebPageServiceInstance.New("PageF1_P2", folder1, true, 200);
            var pageF1_P1 = WebPageServiceInstance.New("PageF1_P1", folder1, false, 100);

            pages.Add(pageF0_P2);
            pages.Add(pageF0_P1);
            pages.Add(pageF0_F1_P1);
            pages.Add(pageF0_F1_P2);
            pages.Add(pageF0_F1_P3);
            pages.Add(pageF1_P2);
            pages.Add(pageF1_P1);

            pages.ForEach(x => WebPageServiceInstance.Save(x));

            return pages;
        }
    }
}
