﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.Admin;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;
using Benday.Cms.Business.Mocks;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{

    /// <summary>
    /// Summary description for FolderListViewFixture
    /// </summary>
    [TestClass]
    public class FolderListViewFixture : AdminUserInterfaceFixture
    {
        public FolderListViewFixture()
        {
            
        }

        private void ClearFolderTestData()
        {
            Folders.Clear();
            RepositoryMock<ICmsFolder> instance = FolderRepositoryInstance as RepositoryMock<ICmsFolder>;
            Assert.IsNotNull(instance, "Instance of folder repository was null.");
            instance.Reset();
        }

        [TestMethod]
        public void FolderListView_InitializeFoldersWithNoExistingFolders()
        {
            Assert.IsNotNull(FolderListViewInstance, "Instance is null.");
            Assert.IsNull(FolderListViewInstance.Folders, "Folders collection should be null.");

            Assert.IsNotNull(Folders, "Test's folders collection should not be null.");

            ClearFolderTestData();

            var rootFolders = from temp in Folders
                              where temp.Parent == null
                              select temp;

            Assert.AreEqual<int>(0, rootFolders.Count(), "Root folder count should not be 0.");

            FolderControllerInstance.InitializeFolders(FolderListViewInstance);

            Assert.IsNotNull(FolderListViewInstance.Folders, "Folder collection should not be null.");
            Assert.AreEqual<int>(0, FolderListViewInstance.Folders.Count, "Folder count didn't match.");
        }

        [TestMethod]
        public void FolderListView_InitializeFoldersSortedByDisplayOrder()
        {
            Assert.IsNotNull(FolderListViewInstance, "Instance is null.");
            Assert.IsNull(FolderListViewInstance.Folders, "Folders collection should be null.");

            Assert.IsNotNull(Folders, "Test's folders collection should not be null.");

            var rootFolders = from temp in Folders
                              where temp.Parent == null
                              select temp;

            Assert.AreNotEqual<int>(0, rootFolders.Count(), "Root folder count should not be 0.");

            FolderControllerInstance.InitializeFolders(FolderListViewInstance);

            Assert.IsNotNull(FolderListViewInstance.Folders, "Folder collection should not be null.");
            Assert.AreEqual<int>(rootFolders.Count(), FolderListViewInstance.Folders.Count, "Folder count didn't match.");

            AssertFoldersOnUi(FolderListViewInstance.Folders);
        }

        private void AssertFoldersOnUi(IList<IFolderInfo> uiFolders)
        {
            int previousDisplayOrderValue = Int32.MinValue;

            ICmsFolder domainFolder;

            foreach (var uiFolder in uiFolders)
            {
                Assert.IsTrue(previousDisplayOrderValue <= uiFolder.DisplayOrder, "Should be sorted by DisplayOrder.");
                previousDisplayOrderValue = uiFolder.DisplayOrder;

                Assert.IsNotNull(uiFolder.Folders, "Subfolders collection should not be null.");

                domainFolder = (from temp in Folders
                                where temp.Id == uiFolder.Id
                                select temp).FirstOrDefault();

                Assert.IsNotNull(domainFolder, "Matching domain folder did not exist.");
                Assert.AreEqual<int>(domainFolder.Folders.Count, uiFolder.Folders.Count, "Folder count between domain and ui version doesn't match.");

                AssertFoldersOnUi(uiFolder.Folders);
            }
        }
        [TestMethod]
        public void FolderListView_ConvertFolderToIFolderInfoParentIsNull()
        {
            var fromFolder = (from temp in Folders
                             where temp.Parent == null
                             select temp).FirstOrDefault();

            Assert.IsNotNull(fromFolder, "fromFolder was null.");

            var toFolder = MockRepositoryInstance.Stub<IFolderInfo>();

            FolderAdapter.Adapt(fromFolder, toFolder);

            UnitTestUtility.AssertAreEqual(fromFolder, toFolder);
        }

        [TestMethod]
        public void FolderListView_ConvertFolderToIFolderInfoParentIsNotNull()
        {
            var fromFolder = (from temp in Folders
                              where temp.Parent != null
                              select temp).FirstOrDefault();

            Assert.IsNotNull(fromFolder, "fromFolder was null.");

            var toFolder = MockRepositoryInstance.Stub<IFolderInfo>();

            FolderAdapter.Adapt(fromFolder, toFolder);

            UnitTestUtility.AssertAreEqual(fromFolder, toFolder);
        }

        
    }
}