using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{

    [TestClass]
    public class LinkViewFixture : AdminUserInterfaceFixture
    {
        public LinkViewFixture()
        {
        }

        protected override void OnTestInitialize()
        {
            Assert.IsNotNull(Folders);
        }

        private ILinkView m_linkViewInstance;
        public ILinkView LinkViewInstance
        {
            get
            {
                if (m_linkViewInstance == null)
                {
                    m_linkViewInstance = MockRepositoryInstance.Stub<ILinkView>();
                }

                return m_linkViewInstance;
            }
        }

        [TestMethod]
        public void LinkView_InitializeBlank()
        {
            LinkControllerInstance.InitializeBlank(LinkViewInstance);

            Assert.AreEqual<int>(default(int), LinkViewInstance.ItemId, "ItemId");
            Assert.AreEqual<string>(String.Empty, LinkViewInstance.Name, "Name should be empty string.");
            Assert.AreEqual<string>(String.Empty, LinkViewInstance.Url, "Url should be empty string.");
            Assert.AreEqual<int>(0, LinkViewInstance.DisplayOrder, "DisplayOrder");
            Assert.AreEqual<int>(default(int), LinkViewInstance.ParentFolderId, "ParentFolderId");
            Assert.AreEqual<bool>(false, LinkViewInstance.IsVisible, "Visible");

            Assert.IsNotNull(LinkViewInstance.Folders, "Folders collection was null.");
            Assert.AreEqual<int>(Folders.Count + 1, LinkViewInstance.Folders.Count, "Folder count didn't match.");
        }

        private void PopulateLinkWithData(ILinkView view)
        {
            view.DisplayOrder = 1;
            view.Url = "this is the url.";
            view.Name = "name";
            view.ParentFolderId = Folders[0].Id;
            view.IsVisible = true;
        }

        [TestMethod]
        public void LinkView_CreateLinkViaUi()
        {
            LinkControllerInstance.InitializeBlank(LinkViewInstance);

            PopulateLinkWithData(LinkViewInstance);

            LinkControllerInstance.Save(LinkViewInstance);

            Assert.AreNotEqual<int>(default(int), LinkViewInstance.ItemId, "ItemId should not be String.Empty after save.");

            var reloaded = LinkServiceInstance.GetById(LinkViewInstance.ItemId);

            UnitTestUtility.AssertAreEqual(reloaded, LinkViewInstance);
        }

        [TestMethod]
        public void LinkView_LoadLinkViaUi()
        {
            LinkControllerInstance.InitializeBlank(LinkViewInstance);

            LinkViewInstance.ItemId = Links[0].Id;

            LinkControllerInstance.LoadById(LinkViewInstance);

            UnitTestUtility.AssertAreEqual(Links[0], LinkViewInstance);
        }

        [TestMethod]
        public void LinkView_ModifyExistingLinkViaUi()
        {
            LinkControllerInstance.InitializeBlank(LinkViewInstance);

            LinkViewInstance.ItemId = Links[0].Id;

            LinkControllerInstance.LoadById(LinkViewInstance);

            PopulateLinkWithData(LinkViewInstance);

            LinkControllerInstance.Save(LinkViewInstance);

            var reloaded = LinkServiceInstance.GetById(LinkViewInstance.ItemId);

            UnitTestUtility.AssertAreEqual(reloaded, LinkViewInstance);
        }

        [TestMethod]
        [ExpectedException(typeof(UnknownIdException))]
        public void LinkView_DeleteLinkViaUi()
        {
            LinkControllerInstance.InitializeBlank(LinkViewInstance);

            LinkViewInstance.ItemId = Links[0].Id;

            LinkControllerInstance.DeleteById(Links[0].Id);

            LinkControllerInstance.LoadById(LinkViewInstance);
        }
    }
}
