using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.UnitTests.UserInterfaceTests
{
    public class UnitTestUtility
    {
        public UnitTestUtility()
        {

        }

        public static void AssertAreEqual(ICmsFileData expected, byte[] actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
            {
                throw new ArgumentNullException("actual", "Argument cannot be null.");
            }

            AssertAreEqual(expected.Data, actual);
        }

        public static void AssertAreEqual(byte[] expected, byte[] actual)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(actual);

            Assert.AreEqual<float>(expected.Length, actual.Length, "Arrays are not the same length.");

            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual<byte>(expected[i], actual[i], "Byte at position {0} did not match.", i);
            }
        }

        public static void AssertAreEqual(byte[] expected, byte[] actual, string message)
        {
            Assert.IsNotNull(expected);
            Assert.IsNotNull(actual);

            string messagePrefix = String.Empty;

            if (message != null && message.Trim().Length > 0)
            {
                messagePrefix = message + ": ";
            }

            Assert.AreEqual<float>(expected.Length, actual.Length, "{0}Arrays are not the same length.", messagePrefix);

            for (int i = 0; i < expected.Length; i++)
            {
                Assert.AreEqual<byte>(expected[i], actual[i], "{0}Byte at position {1} did not match.", messagePrefix, i);
            }
        }

        public static void AssertAreEqual(ICmsFile expected, IFileDetailView actual, bool skipCompareFileData)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.ItemId, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");

            if (expected.Parent == null)
            {
                Assert.AreEqual<int>(default(int), actual.ParentFolderId, "ParentFolderId");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "Visible");

            Assert.AreEqual<string>(expected.Filename, actual.Filename, "Filename");
            Assert.AreEqual<bool>(expected.IsImage, actual.IsImage, "IsImage");

            if (skipCompareFileData == false)
            {
                UnitTestUtility.AssertAreEqual(expected.Filedata, actual.FileData);
            }
        }

        public static void AssertAreEqual(ICmsLink expected, ILinkView actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.ItemId, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");

            if (expected.Parent == null)
            {
                Assert.AreEqual<int>(default(int), actual.ParentFolderId, "ParentFolderId");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "Visible");

            Assert.AreEqual<string>(expected.Url, actual.Url, "Title");
        }

        public static void AssertIsCopy(ICmsWebPage expected, IPageView actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");

            if (expected.Parent == null)
            {
                Assert.AreEqual<int>(default(int), actual.ParentFolderId, "ParentFolderId");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<string>(expected.Content, actual.Html, "Html");
            Assert.AreEqual<string>(expected.Title, actual.PageTitle, "Title");
        }

        public static void AssertAreEqual(ICmsWebPage expected, IPageView actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.ItemId, "Id");    
            
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");

            if (expected.Parent == null)
            {
                Assert.AreEqual<int>(default(int), actual.ParentFolderId, "ParentFolderId");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "Visible");

            Assert.AreEqual<bool>(expected.IsFolderDefault, actual.IsFolderDefault, "IsFolderDefault");
            Assert.AreEqual<string>(expected.Content, actual.Html, "Html");
            Assert.AreEqual<string>(expected.Title, actual.PageTitle, "Title");
        }

        public static void AssertAreEqual(ICmsFolder expected, IFolderView actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.ItemId, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            
            if (expected.Parent == null)
            {
                Assert.AreEqual<int>(default(int), actual.ParentFolderId, "ParentFolderId");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.IsVisible, "Visible");
        }

        public static void AssertAreEqual(ICmsFolder expected, IFolderInfo actual)
        {
            if (expected == null)
                throw new ArgumentNullException("expected", "expected is null.");
            if (actual == null)
                throw new ArgumentNullException("actual", "actual is null.");

            Assert.AreEqual<int>(expected.Id, actual.Id, "Id");
            Assert.AreEqual<string>(expected.Name, actual.Name, "Name");
            Assert.AreEqual<string>(expected.Status, actual.Status, "Status");

            if (expected.Parent == null)
            {
                Assert.IsFalse(Business.BusinessUtility.IsItemIdSaved(actual.ParentFolderId), "ParentFolderId should be null or empty.");
            }
            else
            {
                Assert.AreEqual<int>(expected.Parent.Id, actual.ParentFolderId, "ParentFolderId");
            }

            Assert.AreEqual<bool>(expected.IsVisible, actual.Visible, "Visible");
        }
    }
}
