﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Benday.Cms.WebAndLoadTests
{
    [TestClass]
    public class WcfCmsServiceFixture : UnitTestBase
    {
        public WcfCmsServiceFixture()
        {
        }

        [TestMethod]
        public void WcfCmsService_GetAllFolders()
        {
            GetAllFolders();
        }

        [TestMethod]
        public void WcfCmsService_GetWebPagesByFolderId()
        {
            var folders = GetAllFolders();

            if (folders.Length > 0)
            {
                foreach (var folder in folders)
                {
                    GetWebPagesByFolderId(folder.Id);
                }
            }
        }

        [TestMethod]
        public void WcfCmsService_GetLinksByFolderId()
        {
            var folders = GetAllFolders();

            if (folders.Length > 0)
            {
                foreach (var folder in folders)
                {
                    GetLinksByFolderId(folder.Id);
                }
            }
        }

        [TestMethod]
        public void WcfCmsService_GetWebPageById()
        {
            var allWebPageIds = GetAllWebPageIds();

            foreach (var webPageId in allWebPageIds)
            {
                GetWebPageById(webPageId);
            }
        }

        [TestMethod]
        public void WcfCmsService_CreateNewFolder()
        {
            var folder = CreateUnsavedFolder(0);

            SaveFolder(folder);

            Assert.AreNotEqual<int>(0, folder.Id, "Folder id should not be 0 after save.");
        }

        [TestMethod]
        public void WcfCmsService_CreateNewWebPage()
        {
            var parentFolderId = GetRandomFolderId();

            var webPage = CreateUnsavedWebPage(parentFolderId);

            SaveWebPage(webPage);

            Assert.AreNotEqual<int>(0, webPage.Id, "Web page id should not be 0 after save.");
        }

        [TestMethod]
        public void WcfCmsService_ModifyWebPage()
        {
            var parentFolderId = GetRandomFolderId();

            var webPage = CreateUnsavedWebPage(parentFolderId);

            SaveWebPage(webPage);

            ModifyWebPage(webPage);

            Assert.AreNotEqual<int>(0, webPage.Id, "Web page id should not be 0 after save.");

            SaveWebPage(webPage);            
        }

        private int GetRandomFolderId()
        {
            var allFolders = GetAllFolders();

            Assert.AreNotEqual<int>(0, allFolders.Length, "Should be at least 1 folder.");

            var rnd = new Random();

            var randomIndex = rnd.Next(0, allFolders.Length - 1);

            return allFolders[randomIndex].Id;
        }

        [TestMethod]
        public void WcfCmsService_ModifyFolder()
        {
            var folder = CreateUnsavedFolder(0);

            SaveFolder(folder);

            Assert.AreNotEqual<int>(0, folder.Id, "Folder id should not be 0 after save.");

            var reloaded = GetFolderById(folder.Id);

            ModifyFolder(reloaded);

            SaveFolder(reloaded);

            Assert.AreEqual<int>(folder.Id, reloaded.Id, "Folder id changed when modified.");
        }

        private void ModifyFolder(CmsService.FolderDto folder)
        {
            folder.DisplayOrder += 1;
            folder.IsVisible = !folder.IsVisible;
            folder.Name = UnitTestUtility.GetReasonablyUniqueString("name_");
        }

        private void ModifyWebPage(CmsService.WebPageDto page)
        {
            page.DisplayOrder += 1;
            page.IsVisible = !page.IsVisible;
            page.Name = UnitTestUtility.GetReasonablyUniqueString("name_");

            page.Title = UnitTestUtility.GetReasonablyUniqueString("title_");
            page.Content = UnitTestUtility.GetReasonablyUniqueString("content_");
        }

        private CmsService.WebPageDto CreateUnsavedWebPage(int parentFolderId)
        {
            var page = new CmsService.WebPageDto();

            page.DisplayOrder = 100;
            page.IsVisible = true;
            page.Name = UnitTestUtility.GetReasonablyUniqueString("name_");
            page.ParentId = parentFolderId;

            page.Title = UnitTestUtility.GetReasonablyUniqueString("title_");
            page.Content = UnitTestUtility.GetReasonablyUniqueString("content_");
            
            return page;
        }

        private CmsService.FolderDto CreateUnsavedFolder(int parentFolderId)
        {
            CmsService.FolderDto folder = new CmsService.FolderDto();

            folder.DisplayOrder = 100;
            folder.IsVisible = true;
            folder.Name = UnitTestUtility.GetReasonablyUniqueString("name_");
            folder.ParentId = parentFolderId;

            return folder;
        }

        private CmsService.FolderDto GetFolderById(int id)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                CmsService.FolderDto result = client.GetFolderById(id);

                client.Close();

                return result;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private void SaveWebPage(Benday.Cms.WebAndLoadTests.CmsService.WebPageDto saveThis)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                CmsService.WebPageDto result = client.SaveWebPage(saveThis);

                client.Close();

                saveThis.Id = result.Id;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private void SaveFolder(Benday.Cms.WebAndLoadTests.CmsService.FolderDto saveThis)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                CmsService.FolderDto result = client.SaveFolder(saveThis);

                client.Close();

                saveThis.Id = result.Id;                
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private CmsService.WebPageDto GetWebPageById(int id)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                CmsService.WebPageDto result = client.GetWebPageById(id);

                client.Close();

                return result;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private int[] GetAllWebPageIds()
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                var results = client.GetAllWebPageIds();

                client.Close();

                return results;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private CmsService.WebPageDto[] GetWebPagesByFolderId(int folderId)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                var results = client.GetWebPagesByFolderId(folderId);

                if (results == null)
                {
                    Console.WriteLine("Web page count was 0.");
                }
                else
                {
                    Console.WriteLine(String.Format("Web page count: {0}", results.Length));
                }

                client.Close();

                return results;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private CmsService.LinkDto[] GetLinksByFolderId(int folderId)
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                var results = client.GetLinksByFolderId(folderId);

                if (results == null)
                {
                    Console.WriteLine("Link count was 0.");
                }
                else
                {
                    Console.WriteLine(String.Format("Link count: {0}", results.Length));
                }

                client.Close();

                return results;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

        private CmsService.FolderDto[] GetAllFolders()
        {
            CmsService.CmsServiceClient client = null;

            try
            {
                client = new CmsService.CmsServiceClient();

                CmsService.FolderDto[] results = client.GetFolders();

                if (results == null)
                {
                    Console.WriteLine("Folder count was 0.");
                }
                else
                {
                    Console.WriteLine(String.Format("Folder count: {0}", results.Length));
                }

                client.Close();

                return results;
            }
            catch (Exception)
            {
                if (client != null) client.Abort();
                throw;
            }
        }

    }    
}
