﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;

namespace Benday.Cms.WebUI.Administrator
{
    public partial class FileDetailView : CmsAdminPageBase, IFileDetailView
    {
        private IFileController m_controllerInstance;
        public IFileController ControllerInstance
        {
            get
            {
                if (m_controllerInstance == null)
                {
                    m_controllerInstance = ObjectFactoryInstance.Create<IFileController>();
                }

                return m_controllerInstance;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (this.IsPostBack == false)
            {
                var id = WebUiUtility.GetValueAsInt32(Request["Id"], -1);

                if (id == -1)
                {
                    var parentFolderId = WebUiUtility.GetValueAsInt32(Request["FolderId"], 0);

                    ControllerInstance.InitializeBlank(this);

                    this.ParentFolderId = parentFolderId;
                }
                else
                {
                    this.ItemId = id;
                    ControllerInstance.LoadById(this);
                }
            }
        }


        private void UpdateFileDisplayControls()
        {
            if (m_chkModifyFile.Checked == true)
            {
                m_labelFilename.Font.Strikeout = true;
                m_upload.Visible = true;
            }
            else
            {
                m_labelFilename.Font.Strikeout = false;
                m_labelFilename.Visible = true;
                m_upload.Visible = false;
            }
        }
        protected void m_chkModifyFile_CheckedChanged(object sender, EventArgs e)
        {
            UpdateFileDisplayControls();
        }

        #region IFileDetailView Members

        public string Filename
        {
            get
            {
                if (m_chkModifyFile.Checked == true)
                {
                    return m_upload.FileName;
                }
                else
                {
                    return WebUiUtility.GetValue(m_labelFilename);    
                }                
            }
            set
            {
                WebUiUtility.SetValue(m_labelFilename, value);
            }
        }

        public bool IsImage
        {
            get
            {
                return true;
            }
            set
            {
                
            }
        }

        public bool ModifyFile
        {
            get
            {
                return m_chkModifyFile.Checked;
            }
            set
            {
                m_chkModifyFile.Checked = value;
                UpdateFileDisplayControls();
            }
        }

        public byte[] FileData
        {
            get
            {
                if (m_chkModifyFile.Checked == true)
                {
                    return m_upload.FileBytes;
                }
                else
                {
                    return new byte[] { };
                }
            }
            set
            {
                
            }
        }

        #endregion

        #region IFolderItemView Members

        public int ItemId
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_labelItemId);
            }
            set
            {
                WebUiUtility.SetValue(m_labelItemId, value);
            }
        }

        public string Name
        {
            get
            {
                return WebUiUtility.GetValue(m_textName);
            }
            set
            {
                WebUiUtility.SetValue(m_textName, value);
            }
        }

        public bool IsVisible
        {
            get
            {
                return WebUiUtility.GetValue(m_chkIsVisible);
            }
            set
            {
                WebUiUtility.SetValue(m_chkIsVisible, value);
            }
        }

        public int DisplayOrder
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_textDisplayOrder);
            }
            set
            {
                WebUiUtility.SetValue(m_textDisplayOrder, value);
            }
        }

        public int ParentFolderId
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_lstParentFolder, 0);
            }
            set
            {
                WebUiUtility.SelectListItem(m_lstParentFolder, value.ToString());
            }
        }

        public IList<IFolderInfo> Folders
        {
            get
            {
                return null;
            }
            set
            {
                m_lstParentFolder.Items.Clear();

                foreach (var item in value)
                {
                    m_lstParentFolder.Items.Add(new ListItem(item.FullPath, item.Id.ToString()));
                }
            }
        }


        #endregion

        protected void m_btnDelete_Click(object sender, EventArgs e)
        {
            try
            {
                ControllerInstance.DeleteById(this.ItemId);
                RedirectToPreviousScreenOrParentScreen();
            }
            catch (CannotDeleteFolderException ex)
            {
                WebUiUtility.AddMessage(this.Page, ex.Message);
            }
            catch (Exception ex)
            {
                WebUiUtility.AddMessage(this.Page, ex);
            }
        }

        private void RedirectToPreviousScreenOrParentScreen()
        {
            if (BusinessUtility.IsItemIdSaved(this.ParentFolderId) == false)
            {
                Response.Redirect("FolderListView.aspx");
            }
            else
            {
                Response.Redirect(String.Format("FolderDetailView.aspx?Id={0}", this.ParentFolderId));
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            if (BusinessUtility.IsItemIdSaved(this.ItemId) == true)
            {
                m_linkToItem.NavigateUrl = WebUiUtility.GetUrlToFile(this.ItemId.ToString());
                m_linkToItem.Enabled = true;
            }
            else
            {
                m_linkToItem.Enabled = false;
            }

            base.OnPreRender(e);
        }

        private void DoSave()
        {
            ControllerInstance.Save(this);
            WebUiUtility.AddMessage(this, "Saved.");
            FlushCache();
        }

        protected void m_btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                DoSave();
            }
            catch (Exception ex)
            {
                WebUiUtility.AddMessage(this, ex);
            }
        }

        protected void m_btnCancel_Click(object sender, EventArgs e)
        {
            RedirectToPreviousScreenOrParentScreen();
        }
    }
}
