﻿using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using Benday.Cms.Business.Admin;
using Benday.Cms.Business;
using Benday.Cms.Business.Interfaces;

namespace Benday.Cms.WebUI.Administrator
{
    public partial class FolderDetailView : CmsAdminPageBase, IFolderView
    {
        private IFolderController m_controllerInstance;
        public IFolderController ControllerInstance
        {
            get
            {
                if (m_controllerInstance == null)
                {
                    m_controllerInstance = ObjectFactoryInstance.Create<IFolderController>();
                }

                return m_controllerInstance;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (this.IsPostBack == false)
            {
                var id = WebUiUtility.GetValueAsInt32(Request["Id"], -1);

                if (id == -1)
                {
                    ControllerInstance.InitializeBlank(this);
                }
                else
                {
                    this.ItemId = id;
                    ControllerInstance.LoadById(this);
                }
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            if (BusinessUtility.IsItemIdSaved(this.ItemId) == true)
            {
                string newPageUrl = "~/Administrator/PageDetailView.aspx?Id=-1&FolderId={0}";
                m_linkCreateNewPage.NavigateUrl = String.Format(newPageUrl, this.ItemId);
                m_linkCreateNewPage.Enabled = true;

                m_linkToItem.NavigateUrl = String.Format(m_hiddenLinkToItemUrl.Value, this.ItemId);
                m_linkToItem.Enabled = true;

                string newLinkUrl = "~/Administrator/LinkDetailView.aspx?Id=-1&FolderId={0}";
                m_linkCreateNewLink.NavigateUrl = String.Format(newLinkUrl, this.ItemId);
                m_linkCreateNewLink.Enabled = true;

                /*
                string newFileUrl = "~/Administrator/FileDetailView.aspx?Id=-1&FolderId={0}";
                m_linkCreateNewFile.NavigateUrl = String.Format(newFileUrl, this.ItemId);
                m_linkCreateNewFile.Enabled = true;
                */
            }
            else
            {
                m_linkCreateNewPage.Enabled = false;
                m_linkToItem.Enabled = false;
                m_linkCreateNewLink.Enabled = false;
            }
            
            base.OnPreRender(e);
        }

        private void DoSave()
        {
            ControllerInstance.Save(this);
            // FlushCache();
            WebUiUtility.AddMessage(this, "Saved.");            
        }

        protected void m_btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                DoSave();
            }
            catch (Exception ex)
            {
                WebUiUtility.AddMessage(this, ex);
            }
        }

        protected void m_btnCancel_Click(object sender, EventArgs e)
        {
            RedirectToPreviousScreenOrParentScreen();
        }

        public int ItemId
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_labelItemId);
            }
            set
            {
                WebUiUtility.SetValue(m_labelItemId, value);
            }
        }

        public string Name
        {
            get
            {
                return WebUiUtility.GetValue(m_textName);
            }
            set
            {
                WebUiUtility.SetValue(m_textName, value);
            }
        }

        public bool IsVisible
        {
            get
            {
                return WebUiUtility.GetValue(m_chkVisible);
            }
            set
            {
                WebUiUtility.SetValue(m_chkVisible, value);
            }
        }

        public int ParentFolderId
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_lstParentFolder, 0);
            }
            set
            {
                WebUiUtility.SelectListItem(m_lstParentFolder, value.ToString());
            }
        }

        public IList<IFolderInfo> Folders
        {
            get
            {
                return null;
            }
            set
            {
                m_lstParentFolder.Items.Clear();

                foreach (var item in value)
                {
                    m_lstParentFolder.Items.Add(new ListItem(item.FullPath, item.Id.ToString()));
                }
            }
        }

        public int DisplayOrder
        {
            get
            {
                return WebUiUtility.GetValueAsInt32(m_textDisplayOrder);
            }
            set
            {
                WebUiUtility.SetValue(m_textDisplayOrder, value);
            }
        }

        public IList<IPageInfo> Pages
        {
            get
            {
                return null;
            }
            set
            {
                m_grid.DataSource = value;
                m_grid.DataBind();
            }
        }

        public IList<ILinkInfo> Links
        {
            get
            {
                return null;
            }
            set
            {
                m_gridLinks.DataSource = value;
                m_gridLinks.DataBind();
            }
        }

        /*
        public IList<IFileInfo> Files
        {
            get
            {
                return null;
            }
            set
            {
                m_gridFiles.DataSource = value;
                m_gridFiles.DataBind();
            }
        }
        */

        protected void m_btnDelete_Click(object sender, EventArgs e)
        {
            try
            {
                ControllerInstance.DeleteById(this.ItemId);
                RedirectToPreviousScreenOrParentScreen();
            }
            catch (CannotDeleteFolderException ex)
            {
                WebUiUtility.AddMessage(this.Page, ex.Message);
            }
            catch (Exception ex)
            {
                WebUiUtility.AddMessage(this.Page, ex);
            }
        }

        private void RedirectToPreviousScreenOrParentScreen()
        {
            Response.Redirect("FolderListView.aspx");
        }        
    }
}