﻿using System;
using System.Collections.Generic;
using System.Web.UI.WebControls;
using Benday.Cms.Business.Admin;
using Benday.Cms.Business.Interfaces;

namespace Benday.Cms.WebUI.Administrator
{
    public partial class FolderListView : CmsAdminPageBase, IFolderListView
    {
        private IFolderController m_controllerInstance;
        public IFolderController ControllerInstance
        {
            get
            {
                if (m_controllerInstance == null)
                {
                    m_controllerInstance = ObjectFactoryInstance.Create<IFolderController>();
                }

                return m_controllerInstance;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (this.IsPostBack == false)
            {
                ControllerInstance.InitializeFolders(this);
            }
        }

        private void AddChildNodes(TreeNode parentTreeNode, IFolderInfo parentFolderInfo)
        {
            if (parentFolderInfo.Folders == null || parentFolderInfo.Folders.Count == 0)
            {
                return;
            }

            var children = parentFolderInfo.Folders;

            TreeNode currentTreeNode = null;

            foreach (var folderInfo in children)
            {
                currentTreeNode = GetTreeNodeForFolder(folderInfo);

                parentTreeNode.ChildNodes.Add(currentTreeNode);

                AddChildNodes(currentTreeNode, folderInfo);
            }
        }

        private TreeNode GetTreeNodeForFolder(IFolderInfo folder)
        {
            string nodeText = String.Format("{0} (Order: {1}, Visible: {2})", folder.Name, folder.DisplayOrder, folder.Visible);

            //return new TreeNode(
            //    nodeText,
            //    folder.Id.ToString());

            var navigateUrl = String.Format("./FolderDetailView.aspx?Id={0}", folder.Id);

            return new TreeNode(
                nodeText, folder.Name, String.Empty, navigateUrl, String.Empty);
        }

        private void PopulateTree(IList<IFolderInfo> folderInfos)
        {
            m_tree.Nodes.Clear();

            if (folderInfos == null || folderInfos.Count == 0)
            {
                return;
            }

            IDictionary<string, TreeNode> allTreeNodes = new Dictionary<string, TreeNode>();

            TreeNode currentTreeNode = null;

            foreach (var rootFolder in folderInfos)
            {
                currentTreeNode = GetTreeNodeForFolder(rootFolder);

                allTreeNodes.Add(rootFolder.Id.ToString(), currentTreeNode);

                m_tree.Nodes.Add(currentTreeNode);

                AddChildNodes(currentTreeNode, rootFolder);
            }
        }
        #region IFolderListView Members

        public IList<IFolderInfo> Folders
        {
            get
            {
                return null;
            }
            set
            {
                PopulateTree(value);
            }
        }

        #endregion

        private void HandleSelectedNodeChange()
        {
            var node = m_tree.SelectedNode;

            if (node != null)
            {
                Response.Redirect(String.Format("FolderDetailView.aspx?Id={0}", node.Value));
            }
        }
        protected void m_tree_SelectedNodeChanged(object sender, EventArgs e)
        {
            HandleSelectedNodeChange();
        }
    }
}