﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;

namespace Benday.Cms.WebUI.Administrator.Security
{
    public partial class EditUser : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (this.IsPostBack == false)
            {
                if (String.IsNullOrEmpty(Request["username"]) == true || String.IsNullOrEmpty(Request["username"].Trim()) == true)
                {
                    throw new InvalidOperationException("Missing the 'username' variable.");
                }
                else
                {
                    UsernameToEdit = Request["username"].Trim();
                    m_userToEdit = null;

                    PopulateForm();
                }
            }
        }

        public string UsernameToEdit
        {
            get
            {
                return WebUiUtility.GetValue(m_hiddenUsername);                
            }
            set
            {
                WebUiUtility.SetValue(m_hiddenUsername, value);
            }
        }

        private MembershipUser m_userToEdit;
        public MembershipUser UserToEdit
        {
            get
            {
                if (m_userToEdit == null)
                {
                    m_userToEdit = Membership.GetUser(m_hiddenUsername.Value);
                }
                
                return m_userToEdit;
            }
        }

        private void DoSave()
        {
            if (UserToEdit == null)
            {
                throw new InvalidOperationException(String.Format("Unknown user '{0}'.", UsernameToEdit));
            }

            UserToEdit.Comment = this.Comment;
            UserToEdit.Email = this.EmailAddress;

            Membership.UpdateUser(UserToEdit);

            if (this.IsLockedOut == false)
            {
                UserToEdit.UnlockUser();
            }

            if (this.IsAdministrator == true)
            {
                Roles.AddUserToRole(UserToEdit.UserName, WebUiConstants.ROLE_NAME_ADMINISTRATOR);
            }
            else
            {
                Roles.RemoveUserFromRole(UserToEdit.UserName, WebUiConstants.ROLE_NAME_ADMINISTRATOR);
            }

            Response.Redirect("ListUsers.aspx");
        }

        private void PopulateForm()
        {
            if (UserToEdit == null)
            {
                throw new InvalidOperationException(String.Format("Unknown user '{0}'.", UsernameToEdit));
            }

            this.Comment = UserToEdit.Comment;
            this.CreationDate = UserToEdit.CreationDate;
            this.EmailAddress = UserToEdit.Email;
            this.IsLockedOut = UserToEdit.IsLockedOut;
            this.PasswordQuestion = UserToEdit.PasswordQuestion;
            this.UserName = UserToEdit.UserName;

            this.IsAdministrator = Roles.IsUserInRole(UserToEdit.UserName, WebUiConstants.ROLE_NAME_ADMINISTRATOR);
        }

        public string Comment
        {
            get
            {
                return WebUiUtility.GetValue(m_textComment);
            }
            set
            {
                WebUiUtility.SetValue(m_textComment, value);
            }
        }

        public DateTime CreationDate
        {
            get
            {
                return WebUiUtility.GetValueAsDateTime(m_labelCreationDate);
            }
            set
            {
                WebUiUtility.SetValue(m_labelCreationDate, value);
            }
        }

        public bool IsLockedOut
        {
            get
            {
                return WebUiUtility.GetValue(m_chkIsLockedOut);
            }
            set
            {
                WebUiUtility.SetValue(m_chkIsLockedOut, value);
            }
        }

        public string PasswordQuestion
        {
            get
            {
                return WebUiUtility.GetValue(m_textPasswordQuestion);
            }
            set
            {
                WebUiUtility.SetValue(m_textPasswordQuestion, value);
            }
        }

        public string UserName
        {
            get
            {
                return WebUiUtility.GetValue(m_labelUserName);
            }
            set
            {
                WebUiUtility.SetValue(m_labelUserName, value);
            }
        }

        public string EmailAddress
        {
            get
            {
                return WebUiUtility.GetValue(m_textEmailAddress);
            }
            set
            {
                WebUiUtility.SetValue(m_textEmailAddress, value);
            }
        }

        public bool IsAdministrator
        {
            get
            {
                return WebUiUtility.GetValue(m_chkIsAdministrator);
            }
            set
            {
                WebUiUtility.SetValue(m_chkIsAdministrator, value);
            }
        }

        protected void m_btnSave_Click(object sender, EventArgs e)
        {
            DoSave();
        }

        protected void m_btnCancel_Click(object sender, EventArgs e)
        {
            Response.Redirect("ListUsers.aspx");
        }
        
        
    }
}
