using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Benday.Cms.Business;
using Benday.Cms.Common;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business.Mocks;
using Benday.Cms.Factories;

namespace Benday.Cms.WebUI
{
    public static class BaseClassUtility
    {
        private const string SessionKey_ObjectFactoryInstance = "Cms.ObjectFactoryInstance";

        public static void FlushCache()
        {
            HttpResponse.RemoveOutputCacheItem("/DisplayWebPage.aspx");
        }

        public static IObjectFactory ObjectFactoryInstance
        {
            get
            {
                IObjectFactory instance = ObjectRepositoryStorage as IObjectFactory;

                if (instance == null)
                {
                    instance = new ObjectFactory();

                    if (ConfigurationManager.AppSettings["UseInMemoryMockRepositories"] == "true")
                    {
                        ConfigureObjectFactoryForMockRepositories(instance);                        
                    }
                    else
                    {
                        // use the default repository configuration
                    }

                    ObjectRepositoryStorage = instance;
                }
                else
                {
                    instance = ObjectRepositoryStorage as IObjectFactory;
                }

                return instance;
            }
        }

        private static void ConfigureObjectFactoryForMockRepositories(IObjectFactory instance)
        {
            instance.RegisterInstance<ILogRepository>(new LogRepositoryMock());
            CmsLinkRepositoryMock linkRepositoryInstance = new CmsLinkRepositoryMock();

            CmsWebPageRepositoryMock pageRepositoryInstance = new CmsWebPageRepositoryMock();
            var folderRepositoryInstance = new FolderRepositoryMock(pageRepositoryInstance, linkRepositoryInstance);
            instance.RegisterInstance<ICmsFolderRepository>(folderRepositoryInstance);
            instance.RegisterInstance<ICmsLinkRepository>(linkRepositoryInstance);
            instance.RegisterInstance<ICmsWebPageRepository>(pageRepositoryInstance);
            // instance.RegisterInstance<ICmsFileRepository>(new CmsFileRepositoryMock());
        }

        private static object ObjectRepositoryStorage
        {
            get
            {
                if (ConfigurationManager.AppSettings["ObjectFactoryStorage"] == "SESSION")
                {
                    return HttpContext.Current.Session[SessionKey_ObjectFactoryInstance];
                }
                else
                {
                    return HttpContext.Current.Items[SessionKey_ObjectFactoryInstance];
                }
            }
            set
            {
                if (ConfigurationManager.AppSettings["ObjectFactoryStorage"] == "SESSION")
                {
                    HttpContext.Current.Session[SessionKey_ObjectFactoryInstance] = value;
                }
                else
                {
                    HttpContext.Current.Items[SessionKey_ObjectFactoryInstance] = value;
                }
            }
        }
    }
}
