using System;
using System.Web;
using System.Threading;

namespace Benday.Cms.WebUI.Caching
{
    public class CustomTerribleCacheManager
    {
        private const string TERRIBLE_CACHE_INSTANCE_KEY = "TERRIBLE_CACHE_INSTANCE";
        public static CustomTerribleCacheManager Instance
        {
            get
            {
                HttpApplicationState app = HttpContext.Current.Application;
                var instance = app[TERRIBLE_CACHE_INSTANCE_KEY] as CustomTerribleCacheManager;
                if (instance == null)
                    try
                    {
                        app.Lock();
                        if (app[TERRIBLE_CACHE_INSTANCE_KEY] != null)
                            // someone already initialized it
                            instance = app[TERRIBLE_CACHE_INSTANCE_KEY] as CustomTerribleCacheManager;
                        else
                        {
                            // we have the lock
                            instance = new CustomTerribleCacheManager();
                            app[TERRIBLE_CACHE_INSTANCE_KEY] = instance;
                        }
                        return instance;
                    }
                    finally
                    {
                        app.UnLock();
                    }
                else
                    return instance;
            }
        }

        private CustomTerribleCacheManager()
        {
            m_DataSetInstance = new TerribleCacheDataSet();
            m_SyncRoot = new object();
        }

        private object m_SyncRoot;

        private TerribleCacheDataSet m_DataSetInstance;
        public TerribleCacheDataSet DataSetInstance
        {
            get
            {
                return m_DataSetInstance;
            }
        }

        public bool HasCachedVersionOfPage(string pageId)
        {
            lock(m_SyncRoot)
            {
                Thread.Sleep(250);

                RunGratuitousForLoop();

                System.Data.DataRow[] result =
                   DataSetInstance.WebPageCache.Select(
                   String.Format("Id = '{0}'", pageId));

                if (result == null || result.Length < 1)
                {
                    return false;
                }
                else
                {
                    return true;
                }   
            }            
        }

        private void RunGratuitousForLoop()
        {
            for (int i = 0; i < 10000; i++)
            {
                CallAnotherMethodJustToWasteTime();
            }
        }

        private void CallAnotherMethodJustToWasteTime()
        {
            int value = 0;

            for (int i = 0; i < 10000; i++)
            {
                value += 2;
            }

            Console.WriteLine(value);
        }

        public TerribleCacheDataSet.WebPageCacheRow GetCachedVersionOfPage(
            string pageId)
        {
            lock (m_SyncRoot)
            {
                System.Data.DataRow[] result =
                DataSetInstance.WebPageCache.Select(
                String.Format("Id = '{0}'", pageId));

                if (result == null || result.Length < 1)
                {
                    return null;
                }
                else
                {
                    return result[0] as TerribleCacheDataSet.WebPageCacheRow;
                }
            }
        }

        public void AddPageToCache(string pageId, string name, string title, string displayOrder, string html, string parentFolderId)
        {
            lock (m_SyncRoot)
            {
                if (HasCachedVersionOfPage(pageId) == true)
                {
                    var currentCachedVersion = GetCachedVersionOfPage(pageId);

                    if (currentCachedVersion != null)
                    {
                        DataSetInstance.WebPageCache.Rows.Remove(currentCachedVersion);
                    }
                }

                DataSetInstance.WebPageCache.AddWebPageCacheRow(pageId, name, title, displayOrder, html, parentFolderId);
            }
        }
    }
}
