﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Collections.Generic;
using Benday.Cms.Common;

namespace Benday.Cms.WebUI
{
    [ServiceContract(Namespace = "")]
    [SilverlightFaultBehavior]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class CmsService
    {
        [OperationContract]
        public List<FolderDto> GetFolders()
        {
            var folderDomainService = BaseClassUtility.ObjectFactoryInstance.Create<ICmsFolderService>();

            var fromValues = folderDomainService.GetList();

            List<FolderDto> toValues = new List<FolderDto>();

            Adapt(fromValues, toValues);

            return toValues;
        }

        [OperationContract]
        public FolderDto GetFolderById(int folderId)
        {
            var folderDomainService = BaseClassUtility.ObjectFactoryInstance.Create<ICmsFolderService>();

            var fromValues = folderDomainService.GetList();

            var match = (from temp in fromValues
                         where temp.Id == folderId
                         select temp).FirstOrDefault();

            if (match == null)
            {
                return null;
            }
            else
            {
                var toValue = new FolderDto();

                Adapt(match, toValue);

                return toValue;
            }
        }

        [OperationContract]
        public FolderDto SaveFolder(FolderDto saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            var folderDomainService = BaseClassUtility.ObjectFactoryInstance.Create<ICmsFolderService>();

            ICmsFolder domainFolder = null;

            if (saveThis.Id == 0)
            {
                domainFolder = BaseClassUtility.ObjectFactoryInstance.Create<ICmsFolder>();
            }
            else
            {
                domainFolder = folderDomainService.GetById(saveThis.Id);

                if (domainFolder == null)
                {
                    throw new InvalidOperationException(String.Format("Invalid folder id ('{0}').", saveThis.Id));
                }
            }

            Adapt(saveThis, domainFolder);

            folderDomainService.Save(domainFolder);

            Adapt(domainFolder, saveThis);

            return saveThis;
        }

        [OperationContract]
        public WebPageDto SaveWebPage(WebPageDto saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            var domainService = BaseClassUtility.ObjectFactoryInstance.Create<ICmsWebPageService>();

            ICmsWebPage domainPage = null;

            if (saveThis.Id == 0)
            {
                domainPage = BaseClassUtility.ObjectFactoryInstance.Create<ICmsWebPage>();
            }
            else
            {
                domainPage = domainService.GetById(saveThis.Id);

                if (domainPage == null)
                {
                    throw new InvalidOperationException(String.Format("Invalid page id ('{0}').", saveThis.Id));
                }
            }

            Adapt(saveThis, domainPage);

            domainService.Save(domainPage);

            Adapt(domainPage, saveThis);

            return saveThis;
        }

        private void Adapt(ICmsWebPage fromValue, WebPageDto toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.Name = fromValue.Name;
            toValue.ParentId = fromValue.ParentId;

            toValue.Title = fromValue.Title;
            toValue.Content = fromValue.Content;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;
        }

        private void Adapt(WebPageDto fromValue, ICmsWebPage toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.Name = fromValue.Name;
            toValue.ParentId = fromValue.ParentId;

            toValue.Title = fromValue.Title;
            toValue.Content = fromValue.Content;
            toValue.IsFolderDefault = fromValue.IsFolderDefault;
        }

        private void Adapt(ICmsFolder fromValue, FolderDto toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.Id = fromValue.Id;
            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.Name = fromValue.Name;
            toValue.ParentId = fromValue.ParentId;            
        }

        private void Adapt(FolderDto fromValue, ICmsFolder toValue)
        {
            if (fromValue == null)
                throw new ArgumentNullException("fromValue", "fromValue is null.");
            if (toValue == null)
                throw new ArgumentNullException("toValue", "toValue is null.");

            toValue.DisplayOrder = fromValue.DisplayOrder;
            toValue.IsVisible = fromValue.IsVisible;
            toValue.Name = fromValue.Name;
            toValue.ParentId = fromValue.ParentId;
            toValue.Status = "ACTIVE";
        }

        [OperationContract]
        public List<WebPageDto> GetWebPagesByFolderId(int folderId)
        {
            var domainService = 
                BaseClassUtility.ObjectFactoryInstance.Create<ICmsWebPageService>();

            var allWebPages = domainService.GetList();

            if (allWebPages == null)
            {
                return null;
            }
            else
            {
                var fromValues = (from temp in allWebPages
                                 where temp.ParentId == folderId
                                 select temp);

                var toValues = new List<WebPageDto>();

                foreach (var fromValue in fromValues)
                {
                    var toValue = new WebPageDto(fromValue.Id,
                              fromValue.Name,
                              fromValue.DisplayOrder,
                              fromValue.IsVisible,
                              fromValue.ParentId,
                              fromValue.Title,
                              fromValue.Content,
                              fromValue.IsFolderDefault);

                    toValues.Add(toValue);
                }
                
                return toValues;    
            }            
        }

        [OperationContract]
        public List<int> GetAllWebPageIds()
        {
            var domainService =
                BaseClassUtility.ObjectFactoryInstance.Create<ICmsWebPageService>();

            var allWebPages = domainService.GetList();

            if (allWebPages == null)
            {
                return null;
            }
            else
            {
                var allIds = (from temp in allWebPages
                                  select temp.Id).ToList();

                return allIds;
            }
        }

        [OperationContract]
        public WebPageDto GetWebPageById(int webPageId)
        {
            var domainService =
                BaseClassUtility.ObjectFactoryInstance.Create<ICmsWebPageService>();

            var allWebPages = domainService.GetList();

            if (allWebPages == null)
            {
                return null;
            }
            else
            {
                var fromValue = (from temp in allWebPages
                                  where temp.Id == webPageId
                                  select temp).FirstOrDefault();

                if (fromValue == null)
                {
                    return null;
                }
                else
                {
                    var toValue = new WebPageDto(fromValue.Id,
                              fromValue.Name,
                              fromValue.DisplayOrder,
                              fromValue.IsVisible,
                              fromValue.ParentId,
                              fromValue.Title,
                              fromValue.Content,
                              fromValue.IsFolderDefault);

                    return toValue;
                }
            }
        }

        [OperationContract]
        public List<LinkDto> GetLinksByFolderId(int folderId)
        {
            var domainService =
                BaseClassUtility.ObjectFactoryInstance.Create<ICmsLinkService>();

            var allLinks = domainService.GetList();

            if (allLinks == null)
            {
                return null;
            }
            else
            {
                var fromValues = (from temp in allLinks
                                  where temp.ParentId == folderId
                                  select temp);

                var toValues = new List<LinkDto>();

                foreach (var fromValue in fromValues)
                {
                    var toValue = new LinkDto(fromValue.Id,
                              fromValue.Name,
                              fromValue.DisplayOrder,
                              fromValue.IsVisible,
                              fromValue.ParentId,
                              fromValue.IsFolderDefault,
                              fromValue.Url,
                              fromValue.Target);

                    toValues.Add(toValue);
                }

                return toValues;
            }
        }

        private void Adapt(IList<ICmsFolder> fromValues, List<FolderDto> toValues)
        {
            FolderDto toValue;

            foreach (var fromValue in fromValues)
            {
                toValue = new FolderDto(fromValue.Id, 
                              fromValue.Name, 
                              fromValue.DisplayOrder, 
                              fromValue.IsVisible, 
                              fromValue.ParentId);

                toValues.Add(toValue);
            }
        }
    }
}
