﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Business;
using System.Configuration;
using System.Threading;
using Benday.Cms.Common;
using Benday.Cms.WebUI.Caching;

namespace Benday.Cms.WebUI
{
    public partial class DisplayWebPage : CmsBasePage, IPageDisplayView
    {
        private IPageDisplayController m_controller;
        public IPageDisplayController Controller
        {
            get
            {
                if (m_controller == null)
                {
                    m_controller = ObjectFactoryInstance.Create<IPageDisplayController>();
                }

                return m_controller;
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (String.IsNullOrEmpty(Request["itemId"]) == false)
                {
                    this.ItemKey = ControllerUtility.CreateItemKey("page", Request["itemId"]);
                }
                else if (String.IsNullOrEmpty(Request["folderId"]) == false)
                {
                    this.ItemKey = ControllerUtility.CreateItemKey("folder", Request["folderId"]);
                }
                else if (String.IsNullOrEmpty(Request["linkId"]) == false)
                {
                    this.ItemKey = ControllerUtility.CreateItemKey("link", Request["linkId"]);
                }
                else
                {
                    this.ItemKey = null;
                }

                if (ConfigurationManager.AppSettings["UI.UseTerribleCache"] == "true" && 
                    ItemKey != null && 
                    ItemKey.ItemType == "page")
                {
                    CheckPageCacheAndDisplay();
                }
                else
                {
                    Controller.Display(this);    
                }                
            }
            catch (ThreadAbortException ex)
            {
                Console.WriteLine(ex.ToString());
                // eat the exception caused by response.redirect()
            }
            catch (Exception ex)
            {
                ITrafficLogger logger = ObjectFactoryInstance.Create<ITrafficLogger>();
                logger.LogError("DisplayWebPage.aspx", String.Empty, ex);

                throw;
            }
        }

        private void CheckPageCacheAndDisplay()
        {
            if (CustomTerribleCacheManager.Instance.HasCachedVersionOfPage(
                this.ItemKey.ItemId.ToString()) == true)
            {
                var cachedVersionOfPage =
                    CustomTerribleCacheManager.Instance.GetCachedVersionOfPage(
                    this.ItemKey.ItemId.ToString());

                PageContent = cachedVersionOfPage.Html;
                PageTitle = cachedVersionOfPage.Title;
            }
            else
            {
                Controller.Display(this);
    
                CustomTerribleCacheManager.Instance.AddPageToCache(
                    ItemKey.ItemId.ToString(), 
                    "not really sure", 
                    PageTitle, 
                    "0", 
                    PageContent, 
                    FolderItemKey.ItemId.ToString());
            }            
        }

        private ICmsMasterPage WebPageMaster
        {
            get
            {
                ICmsMasterPage master = this.Master as ICmsMasterPage;

                return master;
            }
        }

        public IItemKey ItemKey { get; set; }

        public void Redirect(string redirectTo)
        {
            Response.Redirect(redirectTo);
        }

        public string PageContent
        {
            get
            {
                return m_litContent.Text;
            }
            set
            {
                m_litContent.Text = value;
            }
        }

        public string PageTitle
        {
            get
            {
                return this.Title;
            }
            set
            {
                this.Title = String.Format("Benjamin Day Consulting, Inc.: {0}", value);
            }
        }

        private IItemKey m_folderItemKey;
        public IItemKey FolderItemKey
        {
            get
            {
                return m_folderItemKey;
            }
            set
            {
                m_folderItemKey = value;

                if (WebPageMaster != null && WebPageMaster.MenuBar != null)
                {
                    WebPageMaster.MenuBar.SelectedFolder = m_folderItemKey;
                }
            }
        }
    }
}