﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Benday.Cms.Business.Interfaces;
using Benday.Cms.Common;

namespace Benday.Cms.WebUI
{
    public partial class HorizontalMenuBar : CmsUserControlBase, ICmsNavigationMenuView
    {
        public IItemKey SelectedFolder
        {
            get;
            set;
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            Initialize();
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            Initialize();
        }

        private void Initialize()
        {
            IMenuController controller = ObjectFactoryInstance.Create<IMenuController>();

            controller.Initialize(this);
        }

        #region ICmsNavigationMenuView Members

        private IList<ICmsNavigationItem> m_folders;
        public IList<ICmsNavigationItem> Folders
        {
            get
            {
                return null;
            }
            set
            {
                m_folders = value;

                m_menuRepeater.DataSource = m_folders;
                m_menuRepeater.DataBind();
            }
        }

        #endregion

        #region ICmsNavigationMenuView Members


        public IList<ICmsNavigationItem> ChildFolders
        {
            get
            {
                return null;
            }
            set
            {

            }
        }

        private IList<ICmsNavigationItem> m_childItems;
        public IList<ICmsNavigationItem> ChildItems
        {
            get
            {
                return m_childItems;
            }
            set
            {
                m_childItems = value;

                if (m_childItems == null || m_childItems.Count <= 1)
                {
                    m_subMenuRepeater.DataSource = null;
                    m_subMenuRepeater.DataBind();

                    m_subMenuRepeater.Visible = true;
                }
                else
                {
                    m_subMenuRepeater.DataSource = m_childItems;
                    m_subMenuRepeater.DataBind();
                    
                    m_subMenuRepeater.Visible = true;
                }                
            }
        }

        #endregion

        private bool IsSelectedFolder(ICmsNavigationItem folder, IItemKey selectedFolderItemKey)
        {
            if (folder == null || selectedFolderItemKey == null)
            {
                return false;
            }
            else
            {
                var folderKey = folder.Key;

                if (folderKey.ItemType == selectedFolderItemKey.ItemType && folderKey.ItemId == selectedFolderItemKey.ItemId)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        protected void m_menuRepeater_ItemDataBound(object sender, RepeaterItemEventArgs e)
        {
            if (m_folders != null)
            {
                HyperLink link = e.Item.FindControl("linkTemplate") as HyperLink;

                if (link != null && e.Item.ItemIndex < m_folders.Count)
                {
                    var folder = m_folders[e.Item.ItemIndex];

                    link.NavigateUrl = folder.Url; // .Replace("DisplayWebPage.aspx", "DisplayWebPage3.aspx");
                    link.Text = folder.Name;

                    if (IsSelectedFolder(folder, this.SelectedFolder) == true)
                    {
                        link.CssClass = "menuItemSelected";
                    }
                }    
            }            
        }

        protected void m_subMenuRepeater_ItemDataBound(object sender, RepeaterItemEventArgs e)
        {
            try
            {
                if (m_childItems != null)
                {
                    HyperLink link = e.Item.FindControl("linkTemplate") as HyperLink;

                    if (link != null && e.Item.ItemIndex < m_childItems.Count)
                    {
                        var navItem = m_childItems[e.Item.ItemIndex];

                        link.NavigateUrl = navItem.Url; //.Replace("DisplayWebPage.aspx", "DisplayWebPage3.aspx");
                        link.Text = navItem.Name;

                        //if (IsSelectedFolder(navItem, this.SelectedFolder) == true)
                        //{
                        //    link.CssClass = "menuItemSelected";
                        //}
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }
        }
    }
}