using System;
using System.Text;
using Benday.Cms.Business.Facades;


namespace Benday.Cms.WebUI
{
	/// <summary>
	/// Summary description for Utility.
	/// </summary>
	public class Utility
	{
		public static bool IsInt32(string p_value)
		{
			try
			{
				Int32.Parse(p_value);

				return true;
			}
			catch
			{
				return false;
			}
		}

		public static bool IsBool(string p_value)
		{
			try
			{
				Boolean.Parse(p_value);

				return true;
			}
			catch
			{
				return false;
			}
		}

		public static bool IsDateTime(string p_value)
		{
			try
			{
				DateTime.Parse(p_value);

				return true;
			}
			catch
			{
				return false;
			}
		}

		public static string ToPascalCase(string p_in)
		{
			string temp =
				p_in.Substring(0, 1).ToUpper() +
				p_in.Substring(1, p_in.Length - 1);

			return temp;
		}

		public static string ToPascalCase(string p_in, bool p_fromOracle)
		{
			if (p_fromOracle == false)
			{
				return ToPascalCase(p_in);
			}
			else
			{
				bool nextCharIsCaps = true;

				StringBuilder sb = new StringBuilder();

				foreach (char character in p_in.ToCharArray())
				{
					if (character != '_')
					{
						if (nextCharIsCaps == true)
						{
							sb.Append(Char.ToUpper(character));
						}
						else
						{
							sb.Append(Char.ToLower(character));
						}

						nextCharIsCaps = false;
					}
					else
					{
						nextCharIsCaps = true;
					}
				}

				string returnValue = sb.ToString();

				return returnValue;
			}
		}

		public static string ToCamelCase(string p_in)
		{
			string temp =
				p_in.Substring(0, 1).ToLower() +
				p_in.Substring(1, p_in.Length - 1);

			return temp;
		}


		public static bool ParseBoolean(object p_value)
		{
			string toLower = p_value.ToString().ToLower();

			if (toLower == "false")
			{
				return false;
			}
			else if (toLower == "true")
			{
				return true;
			}
			else if (toLower == "yes")
			{
				return true;
			}
			else if (toLower == "no")
			{
				return false;
			}
			else
			{
				throw new ArgumentOutOfRangeException(
					"p_value", p_value,
					"Value cannot be parsed to a boolean value.");
			}
		}

		public static bool ContainsUnderscore(string val)
		{
			if (val.IndexOf("_") == -1)
			{
				return false;
			}
			else
			{
				return true;
			}
		}

		public static bool HasMoreThanOneUnderscore(string val)
		{
			if (val == null || val.Trim() == "")
			{
				// no text
				return false;
			}
			else
			{
				int firstIndex = val.IndexOf('_');
				int lastIndex = val.LastIndexOf('_');

				if (firstIndex < 0)
				{
					// no underscore
					return false;
				}
				else
				{
					if (firstIndex != lastIndex)
					{
						// more than one underscore
						return true;
					}
					else
					{
						// only one underscore
						return false;
					}
				}
			}
		}

		public static string RemoveTrailingParenthesis(string input)
		{
			if (input.EndsWith("()") == false)
			{
				return input;
			}
			else
			{
				return input.Substring(0, input.Length - 2);
			}
		}

		public static string SafeToString(object val)
		{
			return SafeToString(val, String.Empty);
		}

		public static string SafeToString(object val, string safeReturnValue)
		{
			if (val == null || val == DBNull.Value)
			{
				return safeReturnValue;
			}
			else
			{
				return val.ToString();
			}
		}
	}
}